@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2002-2005  Thomas Ries <tries@gmx.net>
+    Copyright (C) 2002-2008  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
     
@@ -31,7 +31,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: accessctl.c,v 1.15 2006/05/20 11:48:53 hb9xar Exp $";
+static char const ident[]="$Id: accessctl.c,v 1.18 2008/01/19 16:07:14 hb9xar Exp $";
 
 /* configuration storage */
 struct siproxd_config configuration;
@@ -138,9 +138,7 @@ int process_aclist (char *aclist, struct sockaddr_in from) {
       memset(address,0,sizeof(address));
       memcpy(address,p1,p2-p1);
 
-
       /* mask */
-      p1=strchr(p2,',');
       p1=p2+1;
       p2=strchr(p1,',');
       if (!p2) { /* then this must be the last entry in the list */
@@ -165,6 +163,7 @@ int process_aclist (char *aclist, struct sockaddr_in from) {
       }
 
       mask_int=atoi(mask);
+      if ((mask_int < 0) || (mask_int > 32)) mask_int=32;
       bitmask= (mask_int)? (0xffffffff<<(32-mask_int)) : 0;
 
       DEBUGC(DBCLASS_ACCESS,"check match: entry=%i, filter=%lx, from=%lx", i,
