@@ -1,5 +1,5 @@
-/* -*- Mode: C; c-basic-offset: 3 -*-
-    Copyright (C) 2002-2005  Thomas Ries <tries@gmx.net>
+/*
+    Copyright (C) 2002-2008  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
     
@@ -21,6 +21,7 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <ctype.h>
 #include <time.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,7 +36,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: proxy.c,v 1.95 2006/05/20 11:48:53 hb9xar Exp $";
+static char const ident[]="$Id: proxy.c,v 1.109 2008/01/19 16:07:14 hb9xar Exp $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;	/* defined in siproxd.c */
@@ -110,7 +111,7 @@ int proxy_request (sip_ticket_t *ticket) {
    /*
     * logging of passing calls
     */
-/*&&&& this should be moved to its own loggin plugin */
+/*&&&& this should be moved to its own logging plugin */
    if (configuration.log_calls) {
       osip_uri_t *from_url = NULL;
       osip_uri_t *to_url   = NULL;
@@ -124,7 +125,7 @@ int proxy_request (sip_ticket_t *ticket) {
       if (request->from->url) {
          from_url = request->from->url;
       } else {
-         from_url = (osip_uri_t *)osip_list_get(request->contacts, 0);
+         from_url = (osip_uri_t *)osip_list_get(&(request->contacts), 0);
       }
 
       to_url = request->to->url;
@@ -186,9 +187,7 @@ int proxy_request (sip_ticket_t *ticket) {
        * (rewrite request URI to point to the real host)
        */
       /* 'i' still holds the valid index into the URLMAP table */
-      if (check_rewrite_rq_uri(request) == STS_TRUE) {
-         proxy_rewrite_request_uri(request, i);
-      }
+      proxy_rewrite_request_uri(request, i);
 
       /* if this is CANCEL/BYE request, stop RTP proxying */
       if (MSG_IS_BYE(request) || MSG_IS_CANCEL(request)) {
@@ -201,11 +200,12 @@ int proxy_request (sip_ticket_t *ticket) {
          /* Rewrite the body */
          sts = proxy_rewrite_invitation_body(ticket, DIR_INCOMING);
 
-      } else if (MSG_IS_ACK(request)) {
+      } else if (MSG_IS_ACK(request) || MSG_IS_PRACK(request)) {
          /* Rewrite the body */
          sts = proxy_rewrite_invitation_body(ticket, DIR_INCOMING);
 
       }
+sts=sip_obscure_callid(ticket);
       break;
    
   /*
@@ -243,13 +243,16 @@ int proxy_request (sip_ticket_t *ticket) {
       }
 #endif
 
-      /* rewrite Contact header to represent the masqued address */
+      /* Rewrite Contact header to represent the masqued address */
       sip_rewrite_contact(ticket, DIR_OUTGOING);
 
+      /* Masquerade the User-Agent if configured to do so */
+      proxy_rewrite_useragent(ticket);
+
       /* if an INVITE, rewrite body */
       if (MSG_IS_INVITE(request)) {
          sts = proxy_rewrite_invitation_body(ticket, DIR_OUTGOING);
-      } else if (MSG_IS_ACK(request)) {
+      } else if (MSG_IS_ACK(request) || MSG_IS_PRACK(request)) {
          sts = proxy_rewrite_invitation_body(ticket, DIR_OUTGOING);
       }
 
@@ -260,6 +263,8 @@ int proxy_request (sip_ticket_t *ticket) {
          rtp_stop_fwd(osip_message_get_call_id(request), DIR_OUTGOING);
       }
 
+sts=sip_obscure_callid(ticket);
+
       break;
    
    default:
@@ -298,7 +303,7 @@ int proxy_request (sip_ticket_t *ticket) {
    {
    osip_header_t *max_forwards;
    int forwards_count = DEFAULT_MAXFWD;
-   char mfwd[8];
+   char mfwd[12]; /* 10 digits, +/- sign, termination */
 
    osip_message_get_max_forwards(request, 0, &max_forwards);
    if (max_forwards == NULL) {
@@ -307,6 +312,8 @@ int proxy_request (sip_ticket_t *ticket) {
    } else {
       if (max_forwards->hvalue) {
          forwards_count = atoi(max_forwards->hvalue);
+         if ((forwards_count<=0)||
+             (forwards_count>=LONG_MAX)) forwards_count=DEFAULT_MAXFWD;
          forwards_count -=1;
          osip_free (max_forwards->hvalue);
       }
@@ -390,7 +397,7 @@ int proxy_request (sip_ticket_t *ticket) {
     * If so, fetch address from topmost Route: header and remove it.
     */
    } else if ((type == REQTYP_OUTGOING) && 
-              (request->routes && !osip_list_eol(request->routes, 0))) {
+              (!osip_list_eol(&(request->routes), 0))) {
       sts=route_determine_nexthop(ticket, &sendto_addr, &port);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_request: route_determine_nexthop failed");
@@ -400,7 +407,7 @@ int proxy_request (sip_ticket_t *ticket) {
              utils_inet_ntoa(sendto_addr), port);
 #else
    if ((type == REQTYP_OUTGOING) && 
-              (request->routes && !osip_list_eol(request->routes, 0))) {
+              (!osip_list_eol(&(request->routes), 0))) {
       sts=route_determine_nexthop(ticket, &sendto_addr, &port);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_request: route_determine_nexthop failed");
@@ -431,6 +438,7 @@ int proxy_request (sip_ticket_t *ticket) {
 
       if (url->port) {
          port=atoi(url->port);
+         if ((port<=0) || (port>65535)) port=SIP_PORT;
       } else {
          port=SIP_PORT;
       }
@@ -555,6 +563,9 @@ int proxy_response (sip_ticket_t *ticket) {
 	response->from->url->username? response->from->url->username:"*NULL*",
 	response->from->url->host? response->from->url->host : "*NULL*");
 
+sts=sip_obscure_callid(ticket);
+
+
       /*
        * Response for INVITE - deal with RTP data in body and
        *                       start RTP proxy stream(s). In case
@@ -602,7 +613,7 @@ int proxy_response (sip_ticket_t *ticket) {
        *   response merely indicates that the subscription has been
        *   understood, and that authorization may or may not have been
        *   granted), which then of course is forwarded back to the phone.
-       *   Ans it seems that the Grandstream can *not* *handle* this
+       *   And it seems that the Grandstream can *not* *handle* this
        *   response, as it immediately sends another SUBSCRIBE request.
        *   And this games goes on and on and on...
        *
@@ -633,9 +644,14 @@ int proxy_response (sip_ticket_t *ticket) {
 	     response->from->url->host ? 
                 response->from->url->host : "*NULL*");
 
-      /* rewrite Contact header to represent the masqued address */
+sts=sip_obscure_callid(ticket);
+
+      /* Rewrite Contact header to represent the masqued address */
       sip_rewrite_contact(ticket, DIR_OUTGOING);
 
+      /* Masquerade the User-Agent if configured to do so */
+      proxy_rewrite_useragent(ticket);
+
       /*
        * If an 2xx OK or 1xx response, answer to an INVITE request,
        * rewrite body
@@ -705,7 +721,7 @@ int proxy_response (sip_ticket_t *ticket) {
     * If so, fetch address from topmost Route: header and remove it.
     */
    } else if ((type == RESTYP_OUTGOING) && 
-              (response->routes && !osip_list_eol(response->routes, 0))) {
+              (!osip_list_eol(&(response->routes), 0))) {
       sts=route_determine_nexthop(ticket, &sendto_addr, &port);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_response: route_determine_nexthop failed");
@@ -715,7 +731,7 @@ int proxy_response (sip_ticket_t *ticket) {
              utils_inet_ntoa(sendto_addr), port);
 #else
    if ((type == RESTYP_OUTGOING) && 
-              (response->routes && !osip_list_eol(response->routes, 0))) {
+              (!osip_list_eol(&(response->routes), 0))) {
       sts=route_determine_nexthop(ticket, &sendto_addr, &port);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_response: route_determine_nexthop failed");
@@ -734,7 +750,7 @@ int proxy_response (sip_ticket_t *ticket) {
 #endif
    } else {
       /* get target address and port from VIA header */
-      via = (osip_via_t *) osip_list_get (response->vias, 0);
+      via = (osip_via_t *) osip_list_get (&(response->vias), 0);
       if (via == NULL) {
          ERROR("proxy_response: list_get via failed");
          return STS_FAILURE;
@@ -749,6 +765,7 @@ int proxy_response (sip_ticket_t *ticket) {
 
       if (via->port) {
          port=atoi(via->port);
+         if ((port<=0) || (port>65535)) port=SIP_PORT;
       } else {
          port=SIP_PORT;
       }
@@ -791,6 +808,7 @@ int proxy_rewrite_invitation_body(sip_ticket_t *ticket, int direction){
    sdp_media_t *sdp_med;
    int rtp_direction=0;
    int have_c_media=0;
+   int isrtp = 0 ;
 
    if (configuration.rtp_proxy_enable == 0) return STS_SUCCESS;
 
@@ -799,28 +817,15 @@ int proxy_rewrite_invitation_body(sip_ticket_t *ticket, int direction){
     */
    sts = osip_message_get_body(mymsg, 0, &body);
    if (sts != 0) {
-#if 0
-      if ((MSG_IS_RESPONSE_FOR(mymsg,"INVITE")) &&
-          (MSG_IS_STATUS_1XX(mymsg))) {
-         /* 1xx responses *MAY* contain SDP data */
-         DEBUGC(DBCLASS_PROXY, "rewrite_invitation_body: "
-                "no body found in message");
-         return STS_SUCCESS;
-      } else {
-         /* INVITE request and 200 response *MUST* contain SDP data */
-         ERROR("rewrite_invitation_body: no body found in message");
-         return STS_FAILURE;
-      }
-#else
       DEBUGC(DBCLASS_PROXY, "rewrite_invitation_body: "
                             "no body found in message");
       return STS_SUCCESS;
-#endif
    }
 
    sts = sip_body_to_str(body, &buff, &buflen);
    if (sts != 0) {
       ERROR("rewrite_invitation_body: unable to sip_body_to_str");
+      return STS_FAILURE;
    }
 
    DEBUGC(-1, "rewrite_invitation_body: payload %ld bytes", (long)buflen);
@@ -845,7 +850,7 @@ if (configuration.debuglevel)
    sts = osip_message_get_body(mymsg, 0, &body);
    sts = sip_body_to_str(body, &tmp, &tmplen);
    osip_content_length_to_str(mymsg->content_length, &tmp2);
-   DEBUG("Body before rewrite (clen=%s, strlen=%ld):\n%s\n----",
+   DEBUG("Body before rewrite (may be truncated) - (clen=%s, strlen=%ld):\n%s\n----",
          tmp2, (long)tmplen, tmp);
    osip_free(tmp);
    osip_free(tmp2);
@@ -998,31 +1003,69 @@ if (configuration.debuglevel)
       /* start an RTP proxying stream */
       if (sdp_message_m_port_get(sdp, media_stream_no)) {
          msg_port=atoi(sdp_message_m_port_get(sdp, media_stream_no));
+         if ((msg_port > 0) && (msg_port <= 65535)) {
+            client_id_t client_id;
+            osip_contact_t *contact = NULL;
+            char *tmp=NULL;
+            char *protocol=NULL;
 
-         if (msg_port > 0) {
-            osip_uri_t *cont_url = NULL;
-            char *client_id=NULL;
             /* try to get some additional UA specific unique ID.
-             * Try:
-             * 1) User part of Contact header
-             * 2) Host part of Contact header (will be different
-             *    between internal UA and external UA)
-             * 3) IP address of sender (also different between
-             *    internal and external UA)
+             * This Client-ID should be guaranteed persistent
+             * and not depend on if a UA/Server does include a
+             * particular Header (Contact) or not.
+             * I should just go for the remote IP address here, no?
              */
-            if (!osip_list_eol(mymsg->contacts, 0))
-               cont_url = ((osip_contact_t*)(mymsg->contacts->node->element))->url;
-            if (cont_url) {
-               client_id=cont_url->username;
-               if (client_id == NULL) client_id=cont_url->host;
-            }
-            if (client_id == NULL) {
-               static char from_string[HOSTNAME_SIZE];
-               char *tmp;
-               tmp=utils_inet_ntoa(ticket->from.sin_addr);
-               strncpy(from_string, tmp, HOSTNAME_SIZE-1);
-               from_string[HOSTNAME_SIZE-1]='\0';
-               client_id=from_string;
+            /* &&& NO, using the sender IP will cause troubles if
+             * the remote peer (Proxy/registrar) is thrown out of
+             * the signalling path. Then suddenly the IP address changes.
+             *
+             * I should probably go back to the Contact Header based idea,
+             * with fallback to IP.
+             * I must use a n-item structure, including all things that
+             * can be used for comparing. Then, in the RTP proxy, comparison
+             * must take place according to priorities with the item
+             * with highest priority present in the RTP table *and*
+             * extracted from the SIP message.
+             * To start with, this can be just 2 things, the Contact header
+             * and the Layer 3 IP address. As long as the UAs include the
+             * Contact header, we survive IP changes (e.g. when the SIP
+             * Registrar/Proxy is removed from signalling path within an
+             * ongoing call. This may happen if the Registrar/Proxy does
+             * not explicitely ask to stay in the signalling path using 
+             * Record-Route headers - which is perfectly legal
+             *
+             */
+
+            memset(&client_id, 0, sizeof(client_id));
+
+            /* get the Contact Header if present */
+            osip_message_get_contact(mymsg, 0, &contact);
+            if (contact) osip_contact_to_str(contact, &tmp);
+            if (tmp) strncpy(client_id.contact, tmp, CLIENT_ID_SIZE-1);
+
+            /* store the IP address of the sender */
+            memcpy(&client_id.from_ip, &ticket->from.sin_addr,
+                   sizeof(client_id.from_ip));
+
+            /*
+             * is this an RTP stream ? If yes, set 'isrtp=1'
+             */
+            protocol = sdp_message_m_proto_get (sdp, media_stream_no);
+            if (protocol == NULL) {
+               DEBUGC(DBCLASS_PROXY, "no protocol definition found!");
+            } else {
+               char *check;
+               char *cmp;
+               isrtp = 1;
+               check = protocol ;
+               cmp = "RTP/" ;
+               while (*cmp && (isrtp = isrtp && *check) && 
+                      (isrtp = isrtp && (*cmp++ == toupper(*check++))) ) {} ;
+               if (isrtp) {
+                  DEBUGC(DBCLASS_PROXY, "found RTP protocol [%s]!", protocol);
+               } else {
+                  DEBUGC(DBCLASS_PROXY, "found non RTP protocol [%s]!", protocol);
+               }
             }
 
             /*
@@ -1035,15 +1078,22 @@ if (configuration.debuglevel)
 
             /*
              * Am I running in front of the routing device? Then I cannot
-             * use the external IP to bind a listen socket to, so force
-             * the use of my inbound IP for listening.
+             * use the external IP to bind a listen socket to, but should
+             * use my real IP on the outbound interface (which may legally
+             * be the same as the inbound interface if only one interface
+             * is used).
              */
             if ((rtp_direction == DIR_INCOMING) &&
                 (configuration.outbound_host) &&
                 (strcmp(configuration.outbound_host, "")!=0)) {
                DEBUGC(DBCLASS_PROXY, "proxy_rewrite_invitation_body: "
-                      "in-front-of-NAT-Router");
-               memcpy(&map_addr, &inside_addr, sizeof (map_addr));
+                      "in-front-of-NAT-Router, use real outboud IP");
+               if (get_interface_real_ip(IF_OUTBOUND, &map_addr)
+                   != STS_SUCCESS) {
+                  ERROR("cannot get my real outbound interface address");
+                  /* as we do not know better, take the internal address */
+                  memcpy(&map_addr, &inside_addr, sizeof (map_addr));
+               }
             }
 
             sts = rtp_start_fwd(osip_message_get_call_id(mymsg),
@@ -1051,11 +1101,12 @@ if (configuration.debuglevel)
                                 rtp_direction,
                                 media_stream_no,
                                 map_addr, &map_port,
-                                addr_media, msg_port);
+                                addr_media, msg_port,
+                                isrtp);
 
             if (sts == STS_SUCCESS) {
                /* and rewrite the port */
-               sdp_med=osip_list_get(sdp->m_medias, media_stream_no);
+               sdp_med=osip_list_get(&(sdp->m_medias), media_stream_no);
                if (sdp_med && sdp_med->m_port) {
                   osip_free(sdp_med->m_port);
                   sdp_med->m_port=osip_malloc(8); /* 5 digits, \0 + align */
@@ -1076,7 +1127,7 @@ if (configuration.debuglevel)
    } /* for media_stream_no */
 
    /* remove old body */
-   sts = osip_list_remove(mymsg->bodies, 0);
+   sts = osip_list_remove(&(mymsg->bodies), 0);
    osip_body_free(body);
 
    /* dump new body */
@@ -1108,7 +1159,7 @@ if (configuration.debuglevel)
    sts = osip_message_get_body(mymsg, 0, &body);
    sts = sip_body_to_str(body, &tmp, &tmplen);
    osip_content_length_to_str(mymsg->content_length, &tmp2);
-   DEBUG("Body after rewrite (clen=%s, strlen=%ld):\n%s\n----",
+   DEBUG("Body after rewrite (may be truncated) - (clen=%s, strlen=%ld):\n%s\n----",
          tmp2, (long)tmplen, tmp);
    osip_free(tmp);
    osip_free(tmp2);
@@ -1187,3 +1238,28 @@ int proxy_rewrite_request_uri(osip_message_t *mymsg, int idx){
    }
    return STS_SUCCESS;
 }
+
+
+/*
+ * PROXY_REWRITE_USERAGENT
+ *
+ * rewrites the User Agent String
+ * 
+ * RETURNS
+ *	STS_SUCCESS on success
+ */
+int proxy_rewrite_useragent(sip_ticket_t *ticket){
+   osip_header_t *ua_hdr=NULL;
+
+   osip_message_get_user_agent(ticket->sipmsg, 0, &ua_hdr);
+
+   /* Configured? & Does User-Agent header exist? */
+   if ((configuration.ua_string) && (ua_hdr && ua_hdr->hvalue)) {
+      DEBUGC(DBCLASS_PROXY,"proxy_rewrite_useragent: [%s] -> [%s]",
+             ua_hdr->hvalue, configuration.ua_string);
+      osip_free(ua_hdr->hvalue);
+      ua_hdr->hvalue=osip_malloc(strlen(configuration.ua_string)+1);
+      strcpy(ua_hdr->hvalue, configuration.ua_string);
+   }
+   return STS_SUCCESS;
+}
