@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2002-2005  Thomas Ries <tries@gmx.net>
+    Copyright (C) 2002-2008  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
     
@@ -35,7 +35,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: register.c,v 1.58 2006/05/20 11:48:53 hb9xar Exp $";
+static char const ident[]="$Id: register.c,v 1.62 2008/01/19 16:07:14 hb9xar Exp $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -75,6 +75,7 @@ void register_init(void) {
          for (i=0;i < URLMAP_SIZE; i++) {
             fgets(buff, sizeof(buff), stream);
             sts=sscanf(buff, "***:%i:%i", &urlmap[i].active, &urlmap[i].expires);
+            if (sts == 0) break; /* format error */
             if (urlmap[i].active) {
                osip_uri_init(&urlmap[i].true_url);
                osip_uri_init(&urlmap[i].masq_url);
@@ -89,6 +90,7 @@ void register_init(void) {
                   len  = strlen(buff);\
                   X    =(char*)malloc(len+1);\
                   sts=sscanf(buff,"%s",X);\
+                  if (sts == 0) break;\
                } else {\
                   X = NULL;\
                }\
@@ -109,6 +111,14 @@ void register_init(void) {
             }
          }
          fclose(stream);
+
+         /* check for premature abort of reading the registration file */
+         if (i < URLMAP_SIZE) {
+            /* clean up and delete it */
+            WARN("registration file corrupt, starting with empty table");
+            memset(urlmap, 0, sizeof(urlmap));
+            unlink(configuration.registrationfile);
+         }
       }
    }
    /* initialize save-timer */
@@ -125,6 +135,7 @@ void register_save(void) {
    FILE *stream;
 
    if (configuration.registrationfile) {
+      DEBUGC(DBCLASS_REG,"saving registration table");
       /* write urlmap back to file */
       stream = fopen(configuration.registrationfile, "w+");
       if (!stream) {
@@ -234,7 +245,7 @@ int register_client(sip_ticket_t *ticket, int force_lcl_masq) {
     * Libosip parses an:
     * "Contact: *"
     * the following way (Note: Display name!! and URL is NULL)
-    * (gdb) p *((osip_contact_t*)(sip->contacts->node->element))
+    * (gdb) p *((osip_contact_t*)(sip->contacts.node->element))
     * $5 = {displayname = 0x8af8848 "*", url = 0x0, gen_params = 0x8af8838}
     */
 
@@ -272,19 +283,23 @@ int register_client(sip_ticket_t *ticket, int force_lcl_masq) {
    * look for an Contact expires parameter - in case of REGISTER
    * these two are equal. The Contact expires has higher priority!
    */
-   if (ticket->sipmsg->contacts && ticket->sipmsg->contacts->node &&
-       ticket->sipmsg->contacts->node->element) {
+   if (ticket->sipmsg->contacts.node &&
+       ticket->sipmsg->contacts.node->element) {
       osip_contact_param_get_byname(
-              (osip_contact_t*) ticket->sipmsg->contacts->node->element,
+              (osip_contact_t*) ticket->sipmsg->contacts.node->element,
               EXPIRES, &expires_param);
    }
 
    if (expires_param && expires_param->gvalue) {
       /* get expires from contact Header */
       expires=atoi(expires_param->gvalue);
+      if ((expires < 0) || (expires >= LONG_MAX))
+         expires=configuration.default_expires;
    } else if (expires_hdr && expires_hdr->hvalue) {
       /* get expires from expires Header */
       expires=atoi(expires_hdr->hvalue);
+      if ((expires < 0) || (expires >= LONG_MAX))
+         expires=configuration.default_expires;
    } else {
       char tmp[16];
       /* it seems, the expires field is not present everywhere... */
@@ -331,7 +346,7 @@ int register_client(sip_ticket_t *ticket, int force_lcl_masq) {
                    (url1_contact->host) ? url1_contact->host : "*NULL*",
                    (url2_to->username) ? url2_to->username : "*NULL*",
                    (url2_to->host) ? url2_to->host : "*NULL*",
-                   i, urlmap[i].expires-time_now);
+                   i, (long)urlmap[i].expires-time_now);
             break;
          }
       }
@@ -350,7 +365,7 @@ int register_client(sip_ticket_t *ticket, int force_lcl_masq) {
          urlmap[i].active=1;
          /* Contact: field */
          osip_uri_clone( ((osip_contact_t*)
-                         (ticket->sipmsg->contacts->node->element))->url, 
+                         (ticket->sipmsg->contacts.node->element))->url, 
                          &urlmap[i].true_url);
          /* To: field */
          osip_uri_clone( ticket->sipmsg->to->url, 
@@ -405,7 +420,7 @@ int register_client(sip_ticket_t *ticket, int force_lcl_masq) {
          /* Contact: field (true_url) */
          osip_uri_free(urlmap[i].true_url);
          osip_uri_clone( ((osip_contact_t*)
-                         (ticket->sipmsg->contacts->node->element))->url, 
+                         (ticket->sipmsg->contacts.node->element))->url, 
                          &urlmap[i].true_url);
          /* To: field (reg_url) */
          osip_uri_free(urlmap[i].reg_url);
@@ -505,7 +520,6 @@ void register_agemap(void) {
    /* auto-save of registration table */
    if ((configuration.autosave_registrations > 0) &&
        ((last_save + configuration.autosave_registrations) < t)) {
-      DEBUGC(DBCLASS_REG,"auto-saving registration table");
       register_save();
       last_save = t;
    }
@@ -595,6 +609,7 @@ int register_response(sip_ticket_t *ticket, int flag) {
    /* send answer back */
    if (via->port) {
       port=atoi(via->port);
+      if ((port<=0) || (port>65535)) port=SIP_PORT;
    } else {
       port=configuration.sip_listen_port;
    }
@@ -650,9 +665,13 @@ int register_set_expire(sip_ticket_t *ticket) {
       if (expires_param && expires_param->gvalue) {
          /* get expires from contact Header */
          expires=atoi(expires_param->gvalue);
+         if ((expires < 0) || (expires >= LONG_MAX))
+            expires=configuration.default_expires;
       } else if (expires_hdr && expires_hdr->hvalue) {
          /* get expires from expires Header */
          expires=atoi(expires_hdr->hvalue);
+         if ((expires < 0) || (expires >= LONG_MAX))
+            expires=configuration.default_expires;
       }
 
       DEBUGC(DBCLASS_REG,"Expires=%i, expires_param=%p, expires_hdr=%p",
