@@ -1,18 +1,18 @@
 /*
-    Copyright (C) 2003-2005  Thomas Ries <tries@gmx.net>
+    Copyright (C) 2003-2007  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
-    
+
     Siproxd is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
-    
+
     Siproxd is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warrantry of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-    
+
     You should have received a copy of the GNU General Public License
     along with Siproxd; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
@@ -42,11 +42,15 @@
 #include "rtpproxy.h"
 #include "log.h"
 
+#ifdef GPL
+   #include "dejitter.h"
+#endif
+
 #if !defined(SOL_IP)
 #define SOL_IP IPPROTO_IP
 #endif
 
-static char const ident[]="$Id: rtpproxy_relay.c,v 1.41 2006/06/11 20:29:34 hb9xar Exp $";
+static char const ident[]="$Id: rtpproxy_relay.c,v 1.51 2007/06/08 19:39:03 hb9xar Exp $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -71,11 +75,15 @@ static pthread_t rtpproxy_tid=0;
 static fd_set master_fdset;
 static int    master_fd_max;
 
-/* forward declarations */
-static void *rtpproxy_main(void *i);
-static int rtp_recreate_fdset(void);
-void rtpproxy_kill( void );
+/*
+ * forward declarations of internal functions
+ */
 static void sighdl_alm(int sig) {/* just wake up from select() */};
+static void *rtpproxy_main(void *i);
+static void rtpproxy_kill( void );
+static int  rtp_recreate_fdset(void);
+static void match_socket (int rtp_proxytable_idx);
+static void error_handler (int rtp_proxytable_idx, int socket_type);
 
 
 /*
@@ -89,6 +97,10 @@ int rtp_relay_init( void ) {
    int arg=0;
    struct sigaction sigact;
 
+#ifdef USE_DEJITTER
+   dejitter_init();
+#endif
+
    atexit(rtpproxy_kill);  /* cancel RTP thread at exit */
 
    /* clean proxy table */
@@ -159,26 +171,45 @@ int rtp_relay_init( void ) {
  * main() of rtpproxy
  */
 static void *rtpproxy_main(void *arg) {
-   struct timeval tv;
    fd_set fdset;
    int fd_max;
-   time_t t, last_t=0;
    int i, sts;
    int num_fd;
-   osip_call_id_t callid;
-   static char rtp_buff[RTP_BUFFER_SIZE];
+   static rtp_buff_t rtp_buff;
    int count;
+   struct timeval last_tv ;
+   struct timeval sleep_tv ;
+   struct timeval current_tv ;
+   struct timezone tz ;
 
    memcpy(&fdset, &master_fdset, sizeof(fdset));
    fd_max=master_fd_max;
+   last_tv.tv_sec = 0;
+   last_tv.tv_usec = 0;
 
    /* loop forever... */
    for (;;) {
 
-      tv.tv_sec = 5;
-      tv.tv_usec = 0;
+#ifdef USE_DEJITTER
+      /* calculate time until next packet to send from dejitter buffer */
+      if (!dejitter_delay_of_next_tx(&sleep_tv, &current_tv)) {
+         sleep_tv.tv_sec = 5;
+         sleep_tv.tv_usec = 0;
+      };
+#else
+      sleep_tv.tv_sec = 5;
+      sleep_tv.tv_usec = 0;
+#endif
 
-      num_fd=select(fd_max+1, &fdset, NULL, NULL, &tv);
+      num_fd=select(fd_max+1, &fdset, NULL, NULL, &sleep_tv);
+      gettimeofday(&current_tv, &tz);
+
+#ifdef USE_DEJITTER
+      /* Send delayed Packets that are timed to be send */
+      dejitter_flush(&current_tv, LOCK_FDSET);
+#endif
+
+      /* exit point for this thread in case of program terminaction */
       pthread_testcancel();
       if ((num_fd<0) && (errno==EINTR)) {
          /*
@@ -192,7 +223,6 @@ static void *rtpproxy_main(void *arg) {
          continue;
       }
 
-      time(&t);
 
       /*
        * LOCK the MUTEX
@@ -201,71 +231,78 @@ static void *rtpproxy_main(void *arg) {
 
       /* check for data available and send to destination */
       for (i=0;(i<RTPPROXY_SIZE) && (num_fd>0);i++) {
-         if ( (rtp_proxytable[i].rtp_rx_sock != 0) && 
+         /*
+          * RTCP control socket
+          */
+         if ( (rtp_proxytable[i].rtp_con_rx_sock != 0) &&
+            FD_ISSET(rtp_proxytable[i].rtp_con_rx_sock, &fdset) ) {
+            /* yup, have some data to send */
+            num_fd--;
+
+            /* read from sock rtp_proxytable[i].rtp_con_rx_sock */
+            count=read(rtp_proxytable[i].rtp_con_rx_sock, rtp_buff, RTP_BUFFER_SIZE);
+
+            /* check if something went banana */
+            if (count < 0) error_handler(i,1) ;
+
+            /* Buffer really full? This may indicate a too small buffer! */
+            if (count == RTP_BUFFER_SIZE) {
+               LIMIT_LOG_RATE(30) {
+                  WARN("received an RTCP datagram bigger than buffer size");
+               }
+            }
+
+            /*
+             * forwarding an RTCP packet only makes sense if we really
+             * have got some data in it (count > 0)
+             */
+            if (count > 0) {
+               /* find the corresponding TX socket */
+               if (rtp_proxytable[i].rtp_con_tx_sock == 0) match_socket(i);
+
+               if (rtp_proxytable[i].rtp_con_tx_sock != 0) {
+                  struct sockaddr_in dst_addr;
+
+                  /* write to dest via socket rtp_con_tx_sock */
+                  dst_addr.sin_family = AF_INET;
+                  memcpy(&dst_addr.sin_addr.s_addr,
+                         &rtp_proxytable[i].remote_ipaddr,
+                         sizeof(struct in_addr));
+                  dst_addr.sin_port= htons(rtp_proxytable[i].remote_port+1);
+
+                  /* Don't dejitter RTCP packets */
+                  sts = sendto(rtp_proxytable[i].rtp_con_tx_sock, rtp_buff,
+                               count, 0, (const struct sockaddr *)&dst_addr,
+                               (socklen_t)sizeof(dst_addr));
+                  /* ignore errors here. We don't know if the remote
+                     site does receive RTCP messages at all (or reject
+                     them with ICMP-whatever). If it fails, it is lost.
+                     Basta, end of story. */
+               }
+            } /* count > 0 */
+            /* RTCP does not wind up the keepalive timestamp. */
+         } /* if */
+
+         /*
+          * RTP data stream
+          */
+         if ( (rtp_proxytable[i].rtp_rx_sock != 0) &&
             FD_ISSET(rtp_proxytable[i].rtp_rx_sock, &fdset) ) {
             /* yup, have some data to send */
             num_fd--;
 
-	    /* read from sock rtp_proxytable[i].sock*/
+            /* read from sock rtp_proxytable[i].rtp_rx_sock */
             count=read(rtp_proxytable[i].rtp_rx_sock, rtp_buff, RTP_BUFFER_SIZE);
 
             /* check if something went banana */
-            if (count < 0) {
-               /*
-                * It has been seen on linux 2.2.x systems that for some
-                * reason (ICMP issue? -> below) inside the RTP relay, select()
-                * claims that a certain file descriptor has data available to
-                * read, a subsequent call to read() or recv() then does block!!
-                * So lets make the FD's we are going to use non-blocking, so
-                * we will at least survive and not run into a deadlock.
-                * 
-                * We catch this here with this workaround (pronounce "HACK")
-                * and hope that next time we pass by it will be ok again.
-                */
-               if (errno == EAGAIN) {
-                  /* I may want to remove this WARNing */
-                  WARN("read() [fd=%i, %s:%i] would block, but select() "
-                       "claimed to be readable!",
-                       rtp_proxytable[i].rtp_rx_sock,
-                       utils_inet_ntoa(rtp_proxytable[i].local_ipaddr),
-                       rtp_proxytable[i].local_port);
-                  continue;
-               }
+            if (count < 0) error_handler (i,0);
 
-               /*
-                * I *MAY* receive ICMP destination unreachable messages when I
-                * try to send RTP traffic to a destination that is in HOLD
-                * (better: is not listening on the UDP port where I send
-                * my RTP data to).
-                * So I should *not* do this - or ignore errors originating
-                * by this -> ECONNREFUSED
-                *
-                * Note: This error is originating from a previous send() on the
-                *       same socket and has nothing to do with the read() we have
-                *       done above!
-                */
-               if (errno != ECONNREFUSED) {
-                  /* some other error that I probably want to know about */
-                  int j;
-                  WARN("read() [fd=%i, %s:%i] returned error [%i:%s]",
-                  rtp_proxytable[i].rtp_rx_sock,
-                  utils_inet_ntoa(rtp_proxytable[i].local_ipaddr),
-                  rtp_proxytable[i].local_port, errno, strerror(errno));
-                  for (j=0; j<RTPPROXY_SIZE;j++) {
-                     DEBUGC(DBCLASS_RTP, "%i - rx:%i tx:%i %s@%s dir:%i "
-                            "lp:%i, rp:%i rip:%s",
-                            j,
-                            rtp_proxytable[j].rtp_rx_sock,
-                            rtp_proxytable[j].rtp_tx_sock,
-                            rtp_proxytable[j].callid_number,
-                            rtp_proxytable[j].callid_host,
-                            rtp_proxytable[j].direction,
-                            rtp_proxytable[j].local_port,
-                            rtp_proxytable[j].remote_port,
-                            utils_inet_ntoa(rtp_proxytable[j].remote_ipaddr));
-                  } /* for j */
-              } /* if errno != ECONNREFUSED */
-            } /* count < 0 */
+            /* Buffer really full? This may indicate a too small buffer! */
+            if (count == RTP_BUFFER_SIZE) {
+               LIMIT_LOG_RATE(30) {
+                  WARN("received an RTP datagram bigger than buffer size");
+               }
+            }
 
             /*
              * forwarding an RTP packet only makes sense if we really
@@ -273,95 +310,89 @@ static void *rtpproxy_main(void *arg) {
              */
             if (count > 0) {
                /* find the corresponding TX socket */
-               if (rtp_proxytable[i].rtp_tx_sock == 0) {
-                  int j;
-                  int rtp_direction = rtp_proxytable[i].direction;
-                  int media_stream_no = rtp_proxytable[i].media_stream_no;
-
-                  callid.number = rtp_proxytable[i].callid_number;
-                  callid.host = rtp_proxytable[i].callid_host;
-
-                  for (j=0;(j<RTPPROXY_SIZE);j++) {
-                     char *client_id = rtp_proxytable[i].client_id;
-                     osip_call_id_t cid;
-                     cid.number = rtp_proxytable[j].callid_number;
-                     cid.host = rtp_proxytable[j].callid_host;
-
-                     /* match on:
-                      * - same call ID
-                      * - same media stream
-                      * - opposite direction
-                      * - different client ID
-                      */
-                     if ( (rtp_proxytable[j].rtp_rx_sock != 0) &&
-                          (compare_callid(&callid, &cid) == STS_SUCCESS) &&
-                          (media_stream_no == rtp_proxytable[j].media_stream_no) &&
-                          (rtp_direction != rtp_proxytable[j].direction) &&
-                          (strcmp(rtp_proxytable[j].client_id, client_id) != 0) ) {
-                        rtp_proxytable[i].rtp_tx_sock = rtp_proxytable[j].rtp_rx_sock;
-                        rtp_proxytable[i].opposite_entry=j+1;
-                        DEBUGC(DBCLASS_RTP, "connected entry %i (fd=%i) <-> entry %i (fd=%i)",
-                               j, rtp_proxytable[j].rtp_rx_sock,
-                               i, rtp_proxytable[i].rtp_rx_sock);
-                        break;
-                     }
-                  }
-               } /* rtp_tx_sock == 0 */
+               if (rtp_proxytable[i].rtp_tx_sock == 0) match_socket(i);
 
                if (rtp_proxytable[i].rtp_tx_sock != 0) {
-                  /* write to dest via socket rtp_tx_sock */
                   struct sockaddr_in dst_addr;
+#ifdef USE_DEJITTER
+                  struct timeval ttv;
+#endif
+
+                  /* write to dest via socket rtp_tx_sock */
                   dst_addr.sin_family = AF_INET;
                   memcpy(&dst_addr.sin_addr.s_addr,
-                         &rtp_proxytable[i].remote_ipaddr, 
+                         &rtp_proxytable[i].remote_ipaddr,
                          sizeof(struct in_addr));
                   dst_addr.sin_port= htons(rtp_proxytable[i].remote_port);
 
+#ifdef USE_DEJITTER
+                  dejitter_calc_tx_time(&rtp_buff, &(rtp_proxytable[i].tc),
+                                          &current_tv, &ttv);
+                  dejitter_delayedsendto(rtp_proxytable[i].rtp_tx_sock,
+                                         rtp_buff, count, 0, &dst_addr,
+                                         &ttv, &current_tv,
+                                         &rtp_proxytable[i], NOLOCK_FDSET);
+#else
                   sts = sendto(rtp_proxytable[i].rtp_tx_sock, rtp_buff,
                                count, 0, (const struct sockaddr *)&dst_addr,
                                (socklen_t)sizeof(dst_addr));
                   if (sts == -1) {
                      if (errno != ECONNREFUSED) {
+                        osip_call_id_t callid;
+
                         ERROR("sendto() [%s:%i size=%i] call failed: %s",
                         utils_inet_ntoa(rtp_proxytable[i].remote_ipaddr),
                         rtp_proxytable[i].remote_port, count, strerror(errno));
 
-                    /* if sendto() fails with bad filedescriptor,
-                     * this means that the opposite stream has been
-                     * canceled or timed out.
-                     * we should then cancel this stream as well.*/
+                        /* if sendto() fails with bad filedescriptor,
+                         * this means that the opposite stream has been
+                         * canceled or timed out.
+                         * we should then cancel this stream as well.*/
 
-                    WARN("stopping opposite stream");
-                    /* don't lock the mutex, as we own the lock */
-                    callid.number=rtp_proxytable[i].callid_number;
-                    callid.host=rtp_proxytable[i].callid_host;
-                    rtp_relay_stop_fwd(&callid, rtp_proxytable[i].direction, -1, 1);
+                        WARN("stopping opposite stream");
+                        callid.number=rtp_proxytable[i].callid_number;
+                        callid.host=rtp_proxytable[i].callid_host;
+                        /* don't lock the mutex, as we own the lock already */
+                        sts = rtp_relay_stop_fwd(&callid,
+                                                 rtp_proxytable[i].direction,
+                                                 -1, NOLOCK_FDSET);
+                        if (sts != STS_SUCCESS) {
+                           /* force the streams to timeout on next occasion */
+                           rtp_proxytable[i].timestamp=0;
+                        }
                      }
                   }
+#endif
                }
             } /* count > 0 */
-
             /* update timestamp of last usage for both (RX and TX) entries.
              * This allows silence (no data) on one stream without breaking
              * the connection after the RTP timeout */
-            rtp_proxytable[i].timestamp=t;
+            rtp_proxytable[i].timestamp=current_tv.tv_sec;
             if (rtp_proxytable[i].opposite_entry > 0) {
-               rtp_proxytable[rtp_proxytable[i].opposite_entry-1].timestamp=t;
+               rtp_proxytable[rtp_proxytable[i].opposite_entry-1].timestamp=
+                  current_tv.tv_sec;
             }
-         }
+         } /* if */
       } /* for i */
 
       /*
        * age and clean rtp_proxytable (check every 10 seconds)
        */
-      if (t > (last_t+10) ) {
-         last_t = t;
-	 for (i=0;i<RTPPROXY_SIZE; i++) {
+      if (current_tv.tv_sec > last_tv.tv_sec) {
+         last_tv.tv_sec = current_tv.tv_sec + 10 ;
+         for (i=0;i<RTPPROXY_SIZE; i++) {
             if ( (rtp_proxytable[i].rtp_rx_sock != 0) &&
-		 ((rtp_proxytable[i].timestamp+configuration.rtp_timeout)<t)) {
+                 ((rtp_proxytable[i].timestamp+configuration.rtp_timeout) < 
+                   current_tv.tv_sec)) {
+               osip_call_id_t callid;
+
                /* this one has expired, clean it up */
                callid.number=rtp_proxytable[i].callid_number;
                callid.host=rtp_proxytable[i].callid_host;
+#ifdef USE_DEJITTER
+               dejitter_cancel(&rtp_proxytable[i]);
+#endif
                INFO("RTP stream %s@%s (media=%i) has expired",
                     callid.number, callid.host,
                     rtp_proxytable[i].media_stream_no);
@@ -375,9 +406,10 @@ static void *rtpproxy_main(void *arg) {
                 * This may be a multiple stream conversation (audio/video) and
                 * just one (unused?) has timed out. Seen with VoIPEX PBX! */
                rtp_relay_stop_fwd(&callid, rtp_proxytable[i].direction,
-                                  rtp_proxytable[i].media_stream_no, 1);
-	    }
-	 }
+                                  rtp_proxytable[i].media_stream_no,
+                                  NOLOCK_FDSET);
+            } /* if */
+         } /* for i */
       } /* if (t>...) */
 
       /* copy master FD set */
@@ -401,31 +433,26 @@ static void *rtpproxy_main(void *arg) {
  *	STS_SUCCESS on success
  *	STS_FAILURE on error
  */
-int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
+int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
                          int rtp_direction,
                          int media_stream_no, struct in_addr local_ipaddr,
                          int *local_port, struct in_addr remote_ipaddr,
-                         int remote_port) {
+                         int remote_port, int dejitter) {
    static int prev_used_port = 0;
    int num_ports;
    int i2, i, j;
    int sock, port;
+   int sock_con;
    int freeidx;
    int sts=STS_SUCCESS;
    int tos;
    osip_call_id_t cid;
-   
 
    if (callid == NULL) {
       ERROR("rtp_relay_start_fwd: callid is NULL!");
       return STS_FAILURE;
    }
 
-   if (client_id == NULL) {
-      ERROR("rtp_relay_start_fwd: did not get a client ID!");
-      return STS_FAILURE;
-   }
-
    /*
     * life insurance: check size of received call_id strings
     * I don't know what the maximum allowed size within SIP is,
@@ -444,16 +471,10 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
             callid->host, (long)strlen(callid->host),CALLIDHOST_SIZE);
       return STS_FAILURE;
    }
-   if (client_id && (strlen(client_id) >= CLIENT_ID_SIZE)) {
-      ERROR("rtp_relay_start_fwd: client ID [%s] has too many characters "
-            "(%ld, max=%i)",
-            client_id, (long)strlen(client_id),CLIENT_ID_SIZE);
-      return STS_FAILURE;
-   }
 
    DEBUGC(DBCLASS_RTP,"rtp_relay_start_fwd: starting RTP proxy "
-          "stream for: %s@%s[%s] (%s) #=%i",
-          callid->number, callid->host, client_id,
+          "stream for: CallID=%s@%s [Client-ID=%s] (%s) #=%i",
+          callid->number, callid->host, client_id.contact,
           ((rtp_direction == DIR_INCOMING) ? "incoming RTP" : "outgoing RTP"),
           media_stream_no);
 
@@ -482,14 +503,15 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
          (compare_callid(callid, &cid) == STS_SUCCESS) &&
          (rtp_proxytable[i].direction == rtp_direction) &&
          (rtp_proxytable[i].media_stream_no == media_stream_no) &&
-         (strcmp(rtp_proxytable[i].client_id, client_id) == 0)) {
+         (compare_client_id(rtp_proxytable[i].client_id, client_id) ==
+            STS_SUCCESS)) {
          /*
           * The RTP port number reported by the UA MAY change
           * for a given media stream
           * (seen with KPhone during HOLD/unHOLD)
           * Also the destination IP may change during a re-Invite
           * (seen with Sipphone.com, re-Invites when using
-          * the SIP - POTS gateway [SIP Minutes] 
+          * the SIP - POTS gateway [SIP Minutes]
           */
          /* Port number */
          if (rtp_proxytable[i].remote_port != remote_port) {
@@ -505,19 +527,26 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
             memcpy (&rtp_proxytable[i].remote_ipaddr, &remote_ipaddr,
                      sizeof(remote_ipaddr));
          }
+
+#ifdef USE_DEJITTER
+         /* Initialize up timecrontrol for dejitter function */
+         dejitter_init_time(&rtp_proxytable[i].tc, dejitter);
+#endif
+
+
          /* return the already known local port number */
-         DEBUGC(DBCLASS_RTP,"RTP stream already active (remaddr=%s, "
+         DEBUGC(DBCLASS_RTP,"RTP stream already active idx=%i (remaddr=%s, "
                 "remport=%i, lclport=%i, id=%s, #=%i)",
-                utils_inet_ntoa(remote_ipaddr),
+                i, utils_inet_ntoa(remote_ipaddr),
                 rtp_proxytable[i].remote_port,
                 rtp_proxytable[i].local_port,
                 rtp_proxytable[i].callid_number,
                 rtp_proxytable[i].media_stream_no);
-	 *local_port=rtp_proxytable[i].local_port;
-	 sts = STS_SUCCESS;
+         *local_port=rtp_proxytable[i].local_port;
+         sts = STS_SUCCESS;
 	 goto unlock_and_exit;
-      }
-   }
+      } /* if already active */
+   } /* for */
 
 
    /*
@@ -527,7 +556,7 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
    for (j=0; j<RTPPROXY_SIZE; j++) {
       if (rtp_proxytable[j].rtp_rx_sock==0) {
          freeidx=j;
-	 break;
+         break;
       }
    }
 
@@ -540,11 +569,12 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
 
    /* TODO: randomize the port allocation - start at a random offset to
          search in the allowed port range (so some modulo stuff w/
-	 random start offset 
-	 - for i=x to (p1-p0)+x; p=p0+mod(x,p1-p0) */
+         random start offset
+         - for i=x to (p1-p0)+x; p=p0+mod(x,p1-p0) */
 
    /* find a local port number to use and bind to it */
-   sock=0;
+   sock=0;	/* RTP socket */
+   sock_con=0;	/* RTCP socket */
    port=0;
 
    if ((prev_used_port < configuration.rtp_port_low) ||
@@ -563,31 +593,46 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
 
       for (j=0; j<RTPPROXY_SIZE; j++) {
          /* check if port already in use */
-         if ((memcmp(&rtp_proxytable[j].local_ipaddr,
-	             &local_ipaddr, sizeof(struct in_addr))== 0) &&
-	     (rtp_proxytable[j].local_port == i) ) break;
+         if (memcmp(&rtp_proxytable[j].local_ipaddr,
+                     &local_ipaddr, sizeof(struct in_addr))== 0) {
+            if (rtp_proxytable[j].local_port == i) break;
+            if (rtp_proxytable[j].local_port == i + 1) break;
+            if (rtp_proxytable[j].local_port + 1 == i) break;
+            if (rtp_proxytable[j].local_port + 1 == i + 1) break;
+          }
       }
 
       /* port is available, try to allocate */
       if (j == RTPPROXY_SIZE) {
          port=i;
-         sock=sockbind(local_ipaddr, port, 0);
-         /* if success break, else try further on */
-         if (sock) break;
-      }
-   } /* for i */
-   prev_used_port = port;
+         sock=sockbind(local_ipaddr, port, 0);	/* RTP */
 
-   DEBUGC(DBCLASS_RTP,"rtp_relay_start_fwd: addr=%s, port=%i, sock=%i "
-          "freeidx=%i", utils_inet_ntoa(local_ipaddr), port, sock, freeidx);
+         if (sock) {
+            sock_con=sockbind(local_ipaddr, port+1, 0);	/* RTCP */
+            /* if success break, else try further on */
+            if (sock_con) break;
+            sts = close(sock);
+            DEBUGC(DBCLASS_RTP,"closed socket %i [%i] for RTP stream because "
+                               "cant get pair sts=%i",
+                               sock, i, sts);
+         } /* if sock */
+      } /* if j */
+
+   } /* for i */
+   prev_used_port = port+1;
+
+   DEBUGC(DBCLASS_RTP,"rtp_relay_start_fwd: addr=%s, port=%i, sock=%i, "
+          "freeidx=%i, input data dejitter buffer=%i usec", 
+          utils_inet_ntoa(local_ipaddr), port, sock, freeidx, dejitter);
 
    /* found an unused port? No -> RTP port pool fully allocated */
-   if ((port == 0) || (sock == 0)) {
+   if ((port == 0) || (sock == 0) || (sock_con == 0)) {
       ERROR("rtp_relay_start_fwd: no RTP port available or bind() failed");
       sts = STS_FAILURE;
       goto unlock_and_exit;
    }
 
+   /*&&&: do RTP and RTCP both set DSCP value? */
    /* set DSCP value, need to be ROOT */
    if (configuration.rtp_dscp) {
       int uid,euid;
@@ -619,6 +664,7 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
 
    /* write entry into rtp_proxytable slot (freeidx) */
    rtp_proxytable[freeidx].rtp_rx_sock=sock;
+   rtp_proxytable[freeidx].rtp_con_rx_sock = sock_con;
 
    if (callid->number) {
       strcpy(rtp_proxytable[freeidx].callid_number, callid->number);
@@ -632,11 +678,8 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
       rtp_proxytable[freeidx].callid_host[0]='\0';
    }
 
-   if (client_id) {
-      strcpy(rtp_proxytable[freeidx].client_id, client_id);
-   } else {
-      rtp_proxytable[freeidx].client_id[0]='\0';
-   }
+   /* store the passed Client-ID data */
+   memcpy(&rtp_proxytable[freeidx].client_id, &client_id, sizeof(client_id_t));
 
    rtp_proxytable[freeidx].direction = rtp_direction;
    rtp_proxytable[freeidx].media_stream_no = media_stream_no;
@@ -648,14 +691,25 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
    rtp_proxytable[freeidx].remote_port=remote_port;
    time(&rtp_proxytable[freeidx].timestamp);
 
+#ifdef USE_DEJITTER
+   /* Initialize up timecrontrol for dejitter function */
+   dejitter_init_time(&rtp_proxytable[freeidx].tc, dejitter);
+#endif
+
    *local_port=port;
 
-   /* call to firewall API */
+   /* call to firewall API: RTP port */
    fwapi_start_rtp(rtp_proxytable[freeidx].direction,
                    rtp_proxytable[freeidx].local_ipaddr,
                    rtp_proxytable[freeidx].local_port,
                    rtp_proxytable[freeidx].remote_ipaddr,
                    rtp_proxytable[freeidx].remote_port);
+   /* call to firewall API: RTCP port */
+   fwapi_start_rtp(rtp_proxytable[freeidx].direction,
+                   rtp_proxytable[freeidx].local_ipaddr,
+                   rtp_proxytable[freeidx].local_port + 1,
+                   rtp_proxytable[freeidx].remote_ipaddr,
+                   rtp_proxytable[freeidx].remote_port + 1);
 
    /* prepare FD set for next select operation */
    rtp_recreate_fdset();
@@ -664,6 +718,15 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, char *client_id,
    if (!pthread_equal(rtpproxy_tid, pthread_self()))
       pthread_kill(rtpproxy_tid, SIGALRM);
 
+//&&&
+   DEBUGC(DBCLASS_RTP,"rtp_relay_start_fwd: started RTP proxy "
+          "stream for: CallID=%s@%s [Client-ID=%s] %s #=%i idx=%i",
+          rtp_proxytable[freeidx].callid_number,
+          rtp_proxytable[freeidx].callid_host,
+          rtp_proxytable[freeidx].client_id.contact,
+          ((rtp_proxytable[freeidx].direction == DIR_INCOMING) ? "incoming RTP" : "outgoing RTP"),
+          rtp_proxytable[freeidx].media_stream_no, freeidx);
+
 unlock_and_exit:
    /* unlock mutex */
    pthread_mutex_unlock(&rtp_proxytable_mutex);
@@ -697,9 +760,10 @@ int rtp_relay_stop_fwd (osip_call_id_t *callid,
    }
 
    DEBUGC(DBCLASS_RTP,"rtp_relay_stop_fwd: stopping RTP proxy "
-          "stream for: %s@%s (%s)",
+          "stream for: %s@%s (%s) (nolock=%i)",
           callid->number, callid->host,
-          ((rtp_direction == DIR_INCOMING) ? "incoming" : "outgoing"));
+          ((rtp_direction == DIR_INCOMING) ? "incoming" : "outgoing"),
+          nolock);
 
    /*
     * lock mutex - only if not requested to skip the lock.
@@ -742,30 +806,46 @@ int rtp_relay_stop_fwd (osip_call_id_t *callid,
          (rtp_proxytable[i].direction == rtp_direction) &&
          ((media_stream_no < 0) ||
           (media_stream_no == rtp_proxytable[i].media_stream_no))) {
+         /* close RTP sockets */
          sts = close(rtp_proxytable[i].rtp_rx_sock);
-	 DEBUGC(DBCLASS_RTP,"closed socket %i for RTP stream "
+         DEBUGC(DBCLASS_RTP,"closed socket %i for RTP stream "
                 "%s:%s == %s:%s  (idx=%i) sts=%i",
-	        rtp_proxytable[i].rtp_rx_sock,
-	        rtp_proxytable[i].callid_number,
-	        rtp_proxytable[i].callid_host,
-	        callid->number, callid->host, i, sts);
+                rtp_proxytable[i].rtp_rx_sock,
+                rtp_proxytable[i].callid_number,
+                rtp_proxytable[i].callid_host,
+                callid->number, callid->host, i, sts);
          if (sts < 0) {
             ERROR("Error in close(%i): %s nolock=%i %s:%s\n",
                   rtp_proxytable[i].rtp_rx_sock,
                   strerror(errno), nolock,
                   callid->number, callid->host);
          }
-         /* call to firewall API */
+         /* call to firewall API (RTP port) */
          fwapi_stop_rtp(rtp_proxytable[i].direction,
                    rtp_proxytable[i].local_ipaddr,
                    rtp_proxytable[i].local_port,
                    rtp_proxytable[i].remote_ipaddr,
                    rtp_proxytable[i].remote_port);
+         /* close RTCP socket */
+         sts = close(rtp_proxytable[i].rtp_con_rx_sock);
+         DEBUGC(DBCLASS_RTP,"closed socket %i for RTCP stream sts=%i",
+                rtp_proxytable[i].rtp_con_rx_sock, sts);
+         if (sts < 0) {
+            ERROR("Error in close(%i): %s nolock=%i %s:%s\n",
+                  rtp_proxytable[i].rtp_con_rx_sock,
+                  strerror(errno), nolock,
+                  callid->number, callid->host);
+         }
+         /* call to firewall API (RTCP port) */
+         fwapi_stop_rtp(rtp_proxytable[i].direction,
+                   rtp_proxytable[i].local_ipaddr,
+                   rtp_proxytable[i].local_port + 1,
+                   rtp_proxytable[i].remote_ipaddr,
+                   rtp_proxytable[i].remote_port + 1);
          /* clean up */
          memset(&rtp_proxytable[i], 0, sizeof(rtp_proxytable[0]));
          got_match=1;
       }
- 
    }
 
    /* did not find an active stream... */
@@ -812,10 +892,16 @@ static int rtp_recreate_fdset(void) {
    master_fd_max=-1;
    for (i=0;i<RTPPROXY_SIZE;i++) {
       if (rtp_proxytable[i].rtp_rx_sock != 0) {
+         /* RTP */
          FD_SET(rtp_proxytable[i].rtp_rx_sock, &master_fdset);
-	 if (rtp_proxytable[i].rtp_rx_sock > master_fd_max) {
-	    master_fd_max=rtp_proxytable[i].rtp_rx_sock;
-	 }
+         if (rtp_proxytable[i].rtp_rx_sock > master_fd_max) {
+            master_fd_max=rtp_proxytable[i].rtp_rx_sock;
+         }
+         /* RTPCP */
+         FD_SET(rtp_proxytable[i].rtp_con_rx_sock, &master_fdset);
+         if (rtp_proxytable[i].rtp_con_rx_sock > master_fd_max) {
+            master_fd_max=rtp_proxytable[i].rtp_con_rx_sock;
+         }
       }
    } /* for i */
    return STS_SUCCESS;
@@ -828,7 +914,7 @@ static int rtp_recreate_fdset(void) {
  * RETURNS
  *	-
  */
-void rtpproxy_kill( void ) {
+static void rtpproxy_kill( void ) {
    void *thread_status;
    osip_call_id_t cid;
    int i, sts;
@@ -839,7 +925,8 @@ void rtpproxy_kill( void ) {
          cid.number = rtp_proxytable[i].callid_number;
          cid.host   = rtp_proxytable[i].callid_host;
          sts = rtp_relay_stop_fwd(&cid, rtp_proxytable[i].direction,
-                                  rtp_proxytable[i].media_stream_no, 0);
+                                  rtp_proxytable[i].media_stream_no,
+                                  LOCK_FDSET);
       }
    }
    
@@ -855,3 +942,114 @@ void rtpproxy_kill( void ) {
    return;
 }
 
+
+/*
+ * match_socket
+ * matches and cross connects two rtp_proxytable entries
+ * (corresponds to the two data directions of one RTP stream)
+ */
+static void match_socket (int rtp_proxytable_idx) {
+   int j;
+   int rtp_direction = rtp_proxytable[rtp_proxytable_idx].direction;
+   int media_stream_no = rtp_proxytable[rtp_proxytable_idx].media_stream_no;
+/*chnnel   int channel = rtp_proxytable[rtp_proxytable_idx].channel;*/
+   osip_call_id_t callid;
+
+   callid.number = rtp_proxytable[rtp_proxytable_idx].callid_number;
+   callid.host = rtp_proxytable[rtp_proxytable_idx].callid_host;
+
+   for (j=0;(j<RTPPROXY_SIZE);j++) {
+      osip_call_id_t cid;
+      cid.number = rtp_proxytable[j].callid_number;
+      cid.host = rtp_proxytable[j].callid_host;
+
+      /* match on:
+       * - same call ID
+       * - same media stream
+       * - opposite direction
+       * - different client ID
+       */
+      if ( (rtp_proxytable[j].rtp_rx_sock != 0) &&
+           (compare_callid(&callid, &cid) == STS_SUCCESS) &&
+           (media_stream_no == rtp_proxytable[j].media_stream_no) &&
+           (rtp_direction != rtp_proxytable[j].direction) /* channel: &&
+           (channel == rtp_proxytable[j].channel)*/ ) {
+         rtp_proxytable[rtp_proxytable_idx].rtp_tx_sock = rtp_proxytable[j].rtp_rx_sock;
+         rtp_proxytable[rtp_proxytable_idx].rtp_con_tx_sock = rtp_proxytable[j].rtp_con_rx_sock;
+         DEBUGC(DBCLASS_RTP, "connected entry %i (fd=%i) <-> entry %i (fd=%i)",
+                             j, rtp_proxytable[j].rtp_rx_sock,
+                             rtp_proxytable_idx,
+                             rtp_proxytable[rtp_proxytable_idx].rtp_rx_sock);
+         break;
+      }
+   }
+}
+
+
+/*
+ * error_handler
+ *
+ * rtp_proxytable_idx:	index into the rtp_proxytable array
+ * socket_type: 	1 - RTCP, 0 - RTP
+ */
+static void error_handler (int rtp_proxytable_idx, int socket_type) {
+   /*
+    * It has been seen on linux 2.2.x systems that for some
+    * reason (ICMP issue? -> below) inside the RTP relay, select()
+    * claims that a certain file descriptor has data available to
+    * read, a subsequent call to read() or recv() then does block!!
+    * So lets make the FD's we are going to use non-blocking, so
+    * we will at least survive and not run into a deadlock.
+    *
+    * We catch this here with this workaround (pronounce "HACK")
+    * and hope that next time we pass by it will be ok again.
+    */
+   if (errno == EAGAIN) {
+      /* I may want to remove this WARNing */
+      WARN("read() [fd=%i, %s:%i] would block, but select() "
+           "claimed to be readable!",
+           socket_type ? rtp_proxytable[rtp_proxytable_idx].rtp_rx_sock : 
+                         rtp_proxytable[rtp_proxytable_idx].rtp_con_rx_sock,
+           utils_inet_ntoa(rtp_proxytable[rtp_proxytable_idx].local_ipaddr),
+           rtp_proxytable[rtp_proxytable_idx].local_port + socket_type);
+   }
+
+   /*
+    * I *MAY* receive ICMP destination unreachable messages when I
+    * try to send RTP traffic to a destination that is in HOLD
+    * (better: is not listening on the UDP port where I send
+    * my RTP data to).
+    * So I should *not* do this - or ignore errors originating
+    * by this -> ECONNREFUSED
+    *
+    * Note: This error is originating from a previous send() on the
+    *       same socket and has nothing to do with the read() we have
+    *       done above!
+    */
+   if (errno != ECONNREFUSED) {
+      /* some other error that I probably want to know about */
+      int j;
+      WARN("read() [fd=%i, %s:%i] returned error [%i:%s]",
+          socket_type ? rtp_proxytable[rtp_proxytable_idx].rtp_rx_sock : 
+                        rtp_proxytable[rtp_proxytable_idx].rtp_con_rx_sock,
+          utils_inet_ntoa(rtp_proxytable[rtp_proxytable_idx].local_ipaddr),
+          rtp_proxytable[rtp_proxytable_idx].local_port + socket_type,
+          errno, strerror(errno));
+      for (j=0; j<RTPPROXY_SIZE;j++) {
+         DEBUGC(DBCLASS_RTP, "%i - rx:%i tx:%i %s@%s dir:%i "
+                "lp:%i, rp:%i rip:%s",
+                j,
+                socket_type ? rtp_proxytable[rtp_proxytable_idx].rtp_rx_sock : 
+                              rtp_proxytable[rtp_proxytable_idx].rtp_con_rx_sock,
+                socket_type ? rtp_proxytable[rtp_proxytable_idx].rtp_tx_sock : 
+                              rtp_proxytable[rtp_proxytable_idx].rtp_con_tx_sock,
+                rtp_proxytable[j].callid_number,
+                rtp_proxytable[j].callid_host,
+                rtp_proxytable[j].direction,
+                rtp_proxytable[j].local_port,
+                rtp_proxytable[j].remote_port,
+                utils_inet_ntoa(rtp_proxytable[j].remote_ipaddr));
+      } /* for j */
+   } /* if errno != ECONNREFUSED */
+}
+
