@@ -1,5 +1,5 @@
-/* -*- Mode: C; c-basic-offset: 3 -*-
-    Copyright (C) 2002-2005  Thomas Ries <tries@gmx.net>
+/*
+    Copyright (C) 2002-2008  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
     
@@ -40,10 +40,9 @@
 
 #include "siproxd.h"
 #include "digcalc.h"
-#include "rewrite_rules.h"
 #include "log.h"
 
-static char const ident[]="$Id: sip_utils.c,v 1.45 2006/05/20 11:50:51 hb9xar Exp $";
+static char const ident[]="$Id: sip_utils.c,v 1.56 2008/01/19 16:07:14 hb9xar Exp $";
 
 
 /* configuration storage */
@@ -64,13 +63,6 @@ osip_message_t *msg_make_template_reply (sip_ticket_t *ticket, int code) {
    osip_message_t *response;
    int pos;
 
-   osip_message_init (&response);
-   response->message=NULL;
-   osip_message_set_version (response, osip_strdup ("SIP/2.0"));
-   osip_message_set_status_code (response, code);
-   osip_message_set_reason_phrase (response, 
-                                   osip_strdup(osip_message_get_reason (code)));
-
    if (request->to==NULL) {
       ERROR("msg_make_template_reply: empty To in request header");
       return NULL;
@@ -81,6 +73,18 @@ osip_message_t *msg_make_template_reply (sip_ticket_t *ticket, int code) {
       return NULL;
    }
 
+   osip_message_init (&response);
+   if (response == NULL) {
+      ERROR("msg_make_template_reply: osip_message_init() failed");
+      return NULL;
+   }
+   response->message=NULL;
+   osip_message_set_version (response, osip_strdup ("SIP/2.0"));
+   osip_message_set_status_code (response, code);
+   osip_message_set_reason_phrase (response, 
+                                   osip_strdup(osip_message_get_reason (code)));
+
+
    osip_to_clone (request->to, &response->to);
    osip_from_clone (request->from, &response->from);
 
@@ -90,15 +94,15 @@ osip_message_t *msg_make_template_reply (sip_ticket_t *ticket, int code) {
       osip_contact_t *res_contact = NULL;
       osip_message_get_contact(request, 0, &req_contact);
       if (req_contact) osip_contact_clone (req_contact, &res_contact);
-      if (res_contact) osip_list_add(response->contacts,res_contact,0);
+      if (res_contact) osip_list_add(&(response->contacts),res_contact,0);
    }
 
    /* via headers */
    pos = 0;
-   while (!osip_list_eol (request->vias, pos)) {
+   while (!osip_list_eol (&(request->vias), pos)) {
       char *tmp;
       osip_via_t *via;
-      via = (osip_via_t *) osip_list_get (request->vias, pos);
+      via = (osip_via_t *) osip_list_get (&(request->vias), pos);
       osip_via_to_str (via, &tmp);
 
       osip_message_set_via (response, tmp);
@@ -149,9 +153,9 @@ int check_vialoop (sip_ticket_t *ticket) {
    found_own_via=0;
    pos = 1;	/* for detecting a loop, don't check the first entry 
    		   as this is my own VIA! */
-   while (!osip_list_eol (my_msg->vias, pos)) {
+   while (!osip_list_eol (&(my_msg->vias), pos)) {
       osip_via_t *via;
-      via = (osip_via_t *) osip_list_get (my_msg->vias, pos);
+      via = (osip_via_t *) osip_list_get (&(my_msg->vias), pos);
       sts = is_via_local (via);
       if (sts == STS_TRUE) found_own_via+=1;
       pos++;
@@ -173,7 +177,8 @@ int check_vialoop (sip_ticket_t *ticket) {
 
 /*
  * check if a given osip_via_t is local. I.e. its address is owned
- * by my inbound or outbound interface
+ * by my inbound or outbound interface (or externally defined using
+ * the host_outbound directive
  *
  * RETURNS
  *	STS_TRUE if the given VIA is one of my interfaces
@@ -201,21 +206,33 @@ int is_via_local (osip_via_t *via) {
       }
    }   
 
+   if (via->port) {
+      port=atoi(via->port);
+      if ((port<=0) || (port>65535)) port=SIP_PORT;
+   } else {
+      port=SIP_PORT;
+   }
+
    found=0;
-   for (i=0; i<2; i++) {
-      /*
-       * search my in/outbound interfaces
-       */
-      DEBUGC(DBCLASS_BABBLE,"resolving IP of interface %s",
-             (i==IF_INBOUND)? "inbound":"outbound");
-      if (get_interface_ip(i, &addr_myself) != STS_SUCCESS) {
-         continue;
+   for (i=0; i<3; i++) {
+      if (i < 2) {
+         /* search my in/outbound interfaces */
+         DEBUGC(DBCLASS_BABBLE,"resolving IP of interface %s",
+                (i==IF_INBOUND)? "inbound":"outbound");
+         if (get_interface_ip(i, &addr_myself) != STS_SUCCESS) {
+            continue;
+         }
+      } else {
+         /* check against a possible defined 'host_outbound' */
+         /* defined? */
+         if (configuration.outbound_host == NULL) continue;
+         DEBUGC(DBCLASS_BABBLE,"resolving IP of interface 'host_outbound'");
+         /* lookup */
+         if (get_ip_by_host(configuration.outbound_host,
+                            &addr_myself) != STS_SUCCESS) continue;
       }
 
       /* check the extracted VIA against my own host addresses */
-      if (via->port) port=atoi(via->port);
-      else port=SIP_PORT;
-
       if ( (memcmp(&addr_myself, &addr_via, sizeof(addr_myself))==0) &&
            (port == configuration.sip_listen_port) ) {
          DEBUG("got address match [%s]", utils_inet_ntoa(addr_via));
@@ -356,7 +373,7 @@ int compare_callid(osip_call_id_t *cid1, osip_call_id_t *cid2) {
     */
    if (cid1->host && cid2->host) {
       /* have both hosts */
-      if (strcmp(cid1->host, cid2->host) != 0) goto mismatch;
+      if (strcasecmp(cid1->host, cid2->host) != 0) goto mismatch;
    } else {
       /* at least one host missing, make sure that both are empty */
       if ( (cid1->host && (cid1->host[0]!='\0')) ||
@@ -391,7 +408,7 @@ int is_sipuri_local (sip_ticket_t *ticket) {
    int found;
    struct in_addr addr_uri, addr_myself;
    int port;
-   int i;
+   int i, sts;
 
    if (sip==NULL) {
       ERROR("called is_sipuri_local with NULL sip");
@@ -409,7 +426,12 @@ int is_sipuri_local (sip_ticket_t *ticket) {
 
    if (utils_inet_aton(sip->req_uri->host, &addr_uri) == 0) {
       /* need name resolution */
-      get_ip_by_host(sip->req_uri->host, &addr_uri);
+      sts=get_ip_by_host(sip->req_uri->host, &addr_uri);
+      if (sts == STS_FAILURE) {
+         DEBUGC(DBCLASS_PROXY, "sip_gen_response: cannot resolve via [%s]",
+                sip->req_uri->host);
+         return STS_FALSE;
+      }
    }   
 
    found=0;
@@ -426,6 +448,7 @@ int is_sipuri_local (sip_ticket_t *ticket) {
       /* check the extracted HOST against my own host addresses */
       if (sip->req_uri->port) {
          port=atoi(sip->req_uri->port);
+         if ((port<=0) || (port>65535)) port=SIP_PORT;
       } else {
          port=SIP_PORT;
       }
@@ -443,71 +466,6 @@ int is_sipuri_local (sip_ticket_t *ticket) {
 }
 
 
-/*
- * check if a given request (outbound -> inbound) shall its
- * request URI get rewritten based upon our UA knowledge
- *
- * RETURNS
- *	STS_TRUE if to be rewritten
- *	STS_FALSE otherwise
- */
-int check_rewrite_rq_uri (osip_message_t *sip) {
-   int i, j, sts;
-   int dflidx;
-   osip_header_t *ua_hdr;
-
-   /* get index of default entry */
-   dflidx=(sizeof(RQ_rewrite)/sizeof(RQ_rewrite[0])) - 1;
-
-   /* check fort existence of method */
-   if ((sip==NULL) ||
-       (sip->sip_method==NULL)) {
-      ERROR("check_rewrite_rq_uri: got NULL method");
-      return STS_FALSE;
-   }
-
-   /* extract UA string */
-   osip_message_get_user_agent (sip, 0, &ua_hdr);
-   if ((ua_hdr==NULL) || (ua_hdr->hvalue==NULL)) {
-      DEBUGC(DBCLASS_SIP, "check_rewrite_rq_uri: NULL UA in Header, "
-             "using default");
-      i=dflidx;
-   } else {
-      /* loop through the knowledge base */
-      for (i=0; RQ_rewrite[i].UAstring; i++) {
-         if (strncmp(RQ_rewrite[i].UAstring, ua_hdr->hvalue,
-                    sizeof(RQ_rewrite[i].UAstring))==0) {
-            DEBUGC(DBCLASS_SIP, "got knowledge entry for [%s]",
-                   ua_hdr->hvalue);
-            break;
-         }
-      } /* for i */
-   } /* if ua_hdr */
-
-   for (j=0; RQ_method[j].name; j++) {
-      if (strncmp(RQ_method[j].name,
-                 sip->sip_method, RQ_method[j].size)==0) {
-         if (RQ_rewrite[i].action[j] >= 0) {
-            sts = (RQ_rewrite[i].action[j])? STS_TRUE: STS_FALSE;
-         } else {
-	    sts = (RQ_rewrite[dflidx].action[j])? STS_TRUE: STS_FALSE;
-         }
-         DEBUGC(DBCLASS_SIP, "check_rewrite_rq_uri: [%s:%s, i=%i, j=%i] "
-                "got action %s",
-                (sip && sip->sip_method) ?
-                  sip->sip_method : "*NULL*",
-                (ua_hdr && ua_hdr->hvalue)? ua_hdr->hvalue:"*NULL*",
-                 i, j, (sts==STS_TRUE)? "rewrite":"norewrite");
-         return sts;
-      }
-   } /* for j */
-
-   WARN("check_rewrite_rq_uri: didn't get a hit of the method [%s]",
-        sip->sip_method);
-   return STS_FALSE;
-}
-
-
 /*
  * SIP_GEN_RESPONSE
  *
@@ -539,6 +497,7 @@ int sip_gen_response(sip_ticket_t *ticket, int code) {
    if (via == NULL)
    {
       ERROR("sip_gen_response: Cannot send response - no via field");
+      osip_message_free(response);
       return STS_FAILURE;
    }
 
@@ -552,6 +511,7 @@ int sip_gen_response(sip_ticket_t *ticket, int code) {
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "sip_gen_response: cannot resolve via [%s]",
                 via->host);
+         osip_message_free(response);
          return STS_FAILURE;
       }
    }   
@@ -559,12 +519,14 @@ int sip_gen_response(sip_ticket_t *ticket, int code) {
    sts = sip_message_to_str(response, &buffer, &buflen);
    if (sts != 0) {
       ERROR("sip_gen_response: msg_2char failed");
+      osip_message_free(response);
       return STS_FAILURE;
    }
 
 
    if (via->port) {
       port=atoi(via->port);
+      if ((port<=0) || (port>65535)) port=SIP_PORT;
    } else {
       port=SIP_PORT;
    }
@@ -595,17 +557,31 @@ int sip_add_myvia (sip_ticket_t *ticket, int interface) {
    int sts;
    char branch_id[VIA_BRANCH_SIZE];
    char *myaddr;
+   int  add_rport=0;
 
    if (get_interface_ip(interface, &addr) != STS_SUCCESS) {
       return STS_FAILURE;
    }
 
+   /* evaluate if we need to include an ;rport */
+   /* incoming */
+   if ((configuration.use_rport & 1) && 
+       ((ticket->direction==REQTYP_INCOMING) || 
+        (ticket->direction==RESTYP_INCOMING)) ) {
+        add_rport=1;
+   } else if ((configuration.use_rport & 2) && 
+       ((ticket->direction==REQTYP_OUTGOING) || 
+        (ticket->direction==RESTYP_OUTGOING)) ) {
+        add_rport=1;
+   }
+
    sts = sip_calculate_branch_id(ticket, branch_id);
 
    myaddr=utils_inet_ntoa(addr);
-   sprintf(tmp, "SIP/2.0/UDP %s:%i;branch=%s",
+   sprintf(tmp, "SIP/2.0/UDP %s:%i;branch=%s%s",
            myaddr, configuration.sip_listen_port,
-           branch_id);
+           branch_id,
+           (add_rport)? ";rport":"");
 
    DEBUGC(DBCLASS_BABBLE,"adding VIA:%s",tmp);
 
@@ -615,7 +591,7 @@ int sip_add_myvia (sip_ticket_t *ticket, int interface) {
    sts = osip_via_parse(via, tmp);
    if (sts!=0) return STS_FAILURE;
 
-   osip_list_add(ticket->sipmsg->vias,via,0);
+   osip_list_add(&(ticket->sipmsg->vias),via,0);
 
    return STS_SUCCESS;
 }
@@ -633,7 +609,7 @@ int sip_del_myvia (sip_ticket_t *ticket) {
    int sts;
 
    DEBUGC(DBCLASS_PROXY,"deleting topmost VIA");
-   via = osip_list_get (ticket->sipmsg->vias, 0);
+   via = osip_list_get (&(ticket->sipmsg->vias), 0);
    
    if ( via == NULL ) {
       ERROR("Got empty VIA list - is your UA configured properly?");
@@ -645,7 +621,7 @@ int sip_del_myvia (sip_ticket_t *ticket) {
       return STS_FAILURE;
    }
 
-   sts = osip_list_remove(ticket->sipmsg->vias, 0);
+   sts = osip_list_remove(&(ticket->sipmsg->vias), 0);
    osip_via_free (via);
    return STS_SUCCESS;
 }
@@ -704,7 +680,7 @@ int sip_rewrite_contact (sip_ticket_t *ticket, int direction) {
          }
 
          /* remove old entry */
-         osip_list_remove(sip_msg->contacts,j);
+         osip_list_remove(&(sip_msg->contacts),j);
          osip_contact_to_str(contact, &tmp);
          osip_contact_free(contact);
 
@@ -721,14 +697,14 @@ int sip_rewrite_contact (sip_ticket_t *ticket, int direction) {
             osip_uri_clone(urlmap[i].true_url, &contact->url);
          }
 
-         osip_list_add(sip_msg->contacts,contact,j);
+         osip_list_add(&(sip_msg->contacts),contact,j);
          replaced=1;
       }
 
-   }
+   } /* for j */
 
    if (replaced == 0) {
-      DEBUGC(DBCLASS_PROXY, "no Contact header rewritten");
+      DEBUGC(DBCLASS_PROXY, "no Contact header rewritten!");
       return STS_FAILURE;
    }
 
@@ -784,7 +760,7 @@ int  sip_calculate_branch_id (sip_ticket_t *ticket, char *id) {
     * If it is there, I use THIS branch parameter as input for
     * our hash calculation
     */
-   via = osip_list_get (sip_msg->vias, 0);
+   via = osip_list_get (&(sip_msg->vias), 0);
    if (via == NULL) {
       ERROR("have a SIP message without any via header");
       return STS_FAILURE;
@@ -930,7 +906,7 @@ int  sip_find_outbound_proxy(sip_ticket_t *ticket, struct in_addr *addr,
        * first match wins
        */
       for (i=0; i<configuration.outbound_proxy_domain_name.used; i++) {
-         if (strcmp(configuration.outbound_proxy_domain_name.string[i],
+         if (strcasecmp(configuration.outbound_proxy_domain_name.string[i],
              domain)==0) {
             sts = get_ip_by_host(configuration.outbound_proxy_domain_host.string[i],
                                  addr);
@@ -941,7 +917,7 @@ int  sip_find_outbound_proxy(sip_ticket_t *ticket, struct in_addr *addr,
                return STS_FAILURE;
             }
             *port=atoi(configuration.outbound_proxy_domain_port.string[i]);
-            if (*port == 0) *port = SIP_PORT;
+            if ((*port<=0) || (*port>65535)) *port=SIP_PORT;
 
             return STS_SUCCESS;
 
@@ -992,6 +968,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
    struct sockaddr_in *from;
    osip_message_t *request;
    osip_message_t *response;
+   struct in_addr tmp_addr;
 
    from=&ticket->from;
    request=ticket->sipmsg;
@@ -1005,8 +982,6 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
     * -> it must be an OUTGOING request
     */
    for (i=0; i<URLMAP_SIZE; i++) {
-      struct in_addr tmp_addr;
-
       if (urlmap[i].active == 0) continue;
       if (get_ip_by_host(urlmap[i].true_url->host, &tmp_addr) == STS_FAILURE) {
          DEBUGC(DBCLASS_SIP, "sip_find_direction: cannot resolve host [%s]",
@@ -1073,7 +1048,6 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
       } /* for i */
    } /* if type == 0 */
 
-
    if (MSG_IS_RESPONSE(ticket->sipmsg)) {
       /* &&&& Open Issue &&&&
          it has been seen with cross-provider calls that the FROM may be 'garbled'
@@ -1083,13 +1057,13 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
          Maybe I should put in a 'siproxd' ftag value to recognize it as a header
          inserted by myself
       */
-      if ((type == 0) && (!osip_list_eol(response->vias, 0))) {
+      if ((type == 0) && (!osip_list_eol(&(response->vias), 0))) {
          osip_via_t *via;
          struct in_addr addr_via, addr_myself;
          int port_via, port_ua;
 
          /* get the via address */
-         via = (osip_via_t *) osip_list_get (response->vias, 0);
+         via = (osip_via_t *) osip_list_get (&(response->vias), 0);
          DEBUGC(DBCLASS_SIP, "sip_find_direction: check via [%s] for "
                 "registered UA",via->host);
          sts=get_ip_by_host(via->host, &addr_via);
@@ -1110,12 +1084,12 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
 
                port_via=0;
                if (via->port) port_via=atoi(via->port);
-               if (port_via <= 0) port_via=SIP_PORT;
+               if ((port_via<=0) || (port_via>65535)) port_via=SIP_PORT;
 
                port_ua=0;
                if (urlmap[i].true_url->port)
                   port_ua=atoi(urlmap[i].true_url->port);
-               if (port_ua <= 0) port_ua=SIP_PORT;
+               if ((port_ua<=0) || (port_ua>65535)) port_ua=SIP_PORT;
 
                DEBUGC(DBCLASS_SIP, "sip_find_direction: checking for registered "
                       "host [%s:%i] <-> [%s:%i]",
@@ -1132,6 +1106,26 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
       } /* if type == 0 */
    } /* is response */
 
+   /*
+    * if the telegram is received from 127.0.0.1 use my inbound IP as sender,
+    * this likely is a locally REDIRECTED/DNATed (by iptables) packet.
+    * So it is a local UA.
+    * Example Scenario:
+    * Softphone(or PBX) running on the same host as siproxd is running.
+    * Using iptables, you do a REDIRECT of outgoping SIP traffix of the
+    * PBX to be passed to siproxd.
+    */
+   sts=get_interface_ip(IF_INBOUND, &tmp_addr);
+   if ((htonl(from->sin_addr.s_addr) == INADDR_LOOPBACK) ||
+       (from->sin_addr.s_addr == tmp_addr.s_addr)) {
+      if (MSG_IS_REQUEST(ticket->sipmsg)) {
+         type=REQTYP_OUTGOING;
+      } else {
+         type=RESTYP_OUTGOING;
+      }
+   }
+
+
    if (type == 0) {
       DEBUGC(DBCLASS_SIP, "sip_find_direction: unable to determine "
                           "direction of SIP packet");
@@ -1144,6 +1138,205 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
 }
 
 
+/*
+ * SIP_FIXUP_ALERT_INFO
+ *
+ * Asterisk Hack.
+ * Asterisk seems to include broken Alert-Info headers (without <>)
+ * that cause parsing to fail (libosip2).
+ * This function does try to guess if we have such a broken SIP
+ * message and does simply remove the offending Alert-Info header.
+ *
+ * RETURNS
+ *	STS_SUCCESS on success
+ */
+int  sip_fixup_asterisk(char *buff, int *buflen) {
+   char *alert_info_ptr=NULL;
+   /*
+    * Check for Asterisk UA string
+    * User-Agent: Asterisk PBX
+    */
+   if (strstr(buff, "\r\nUser-Agent: Asterisk PBX\r\n")==NULL)
+      return STS_SUCCESS;
+
+   DEBUGC(DBCLASS_SIP, "sip_fixup_alert_info: SIP message is from Asterisk");
+   /*
+    * Look form Alert-Info: header
+    */
+   alert_info_ptr=strstr(buff, "\r\nAlert-Info: ");
+   if (alert_info_ptr) {
+      char *eol_ptr=NULL;
+      DEBUGC(DBCLASS_SIP, "sip_fixup_alert_info: Asterisk message "
+             "with Alert-Info found");
+      eol_ptr=strstr((alert_info_ptr+2), "\r\n");
+      if (eol_ptr) {
+         int i;
+         // cut the Alert-Info header from the message
+         // (actually move the rest of the message up)
+         DEBUGC(DBCLASS_SIP, "sip_fixup_alert_info: removed malformed "
+                "Alert-Info header");
+         for (i=0; i<(*buflen - (eol_ptr - buff)); i++) {
+            alert_info_ptr[i]=eol_ptr[i];
+         } /* for */
+
+         *buflen=strlen(buff);
+      } /* if */
+   }
+   return STS_SUCCESS;
+}
 
 
+/*
+ * SIP_OBSCURE_CALLID
+ *
+  27-May-2007:  - new feature: "Obscure Loops" does modify the Call-IDs in
+                  outgoing requests and thus allows incoming calls forked
+                  off such an outgoing call (redirect, transfer, ...) back
+                  to the same UA where the initial call did originate.
+                  This even seems to fix some issues with Asterisks
+                  Loop detection... :-)
+Needs more thinking. The logic about call termination is quite tricky -
+calls can be terminated from either side... It is not stratigh forward
+when and how to modify CIDs for call cancellation...
+ * ...
+ *
+ * RETURNS
+ *	STS_SUCCESS on success
+ */
+int  sip_obscure_callid(sip_ticket_t *ticket) {
+   static char myident[]="-siproxd";
+   int myidentlen=sizeof(myident)-1; /* a static strlen() of myident */
+
+   osip_message_t *sipmsg;
+   osip_from_t *from;
+   osip_to_t *to;
+   osip_uri_param_t *fromtag, *totag; /* gname, gvalue */
+
+   osip_call_id_t *CID=NULL;
+
+   char *tmp, *tmp2;
+
+
+   /* feature enabled? */
+   if (!configuration.obscure_loops) return STS_SUCCESS;
+
+   sipmsg=ticket->sipmsg;
+
+
+      if (MSG_IS_REQUEST(sipmsg) &&
+          MSG_IS_REGISTER(sipmsg)) return STS_SUCCESS;
+      if (MSG_IS_RESPONSE(sipmsg) &&
+          MSG_IS_RESPONSE_FOR(sipmsg,"REGISTER")) return STS_SUCCESS;
+
+   from=sipmsg->from;
+   osip_from_get_tag(from, &fromtag);
+
+   to=sipmsg->to;
+   osip_to_get_tag(to, &totag);
+
+   /* at lest the From Tag must be present, otherwise out Logic does
+      not work. Is MANDATORY according to RFC3261 */
+   if (!fromtag || !fromtag->gvalue || strlen(fromtag->gvalue)==0) {
+      WARN("sip_obscure_callid: no From-Tag, not RFC3261 conform!");
+      return STS_FAILURE;
+   }
+
+   CID=osip_message_get_call_id(sipmsg);
+
+   /* does CID->number exist? According to RFC3261, this part is mandatory */
+   if (!CID || !CID->number) {
+      WARN("sip_obscure_callid: invalid Call-ID received, not RFC3261 conform!");
+      return STS_FAILURE;
+   }
+
+   DEBUGC(DBCLASS_PROXY, "sip_obscure_callid: current Callid#=%s Direction=%i",
+                         CID->number, ticket->direction);
+
+   switch (ticket->direction) {
+
+   /* Outgoing Request */
+   case REQTYP_OUTGOING:
+   /* also need testing for "normal" incoming call that is present and
+      now we SEND a cancel for this call */
+      tmp=strstr(CID->number, myident);
+      if (tmp==NULL) {
+         /* no obscuring present yet, must be a new call, modify it */
+         tmp=osip_malloc(strlen(CID->number) + myidentlen + 
+                         strlen(fromtag->gvalue) + 1);
+         sprintf(tmp,"%s%s%s",CID->number,myident,fromtag->gvalue);
+         osip_free(CID->number);
+         CID->number=tmp;
+      } else {
+         /* Obscuring is present, do nothing. */
+      }
+      break;
+
+   /* Incoming Response */
+   case RESTYP_INCOMING:
+      tmp=strstr(CID->number,myident);
+      /* modify it back if existing */
+      if (tmp) {
+         /* make sure to cut only the last marker - in case
+            of multiple siproxd instances...
+            I dont know if this actually would work (multiple instances...)*/
+         for (;(tmp2=strstr(tmp+1, myident)) != NULL;) {
+            tmp=tmp2;
+         }
+         tmp[0]='\0';
+      }
+      break;
+
+   /* Incoming Request */
+   case REQTYP_INCOMING:
+      tmp=strstr(CID->number, myident);
+      if (tmp==NULL) {
+      /* no obscuring present yet, must be a new incoming call, do nothing */
+      } else {
+         /* if BYE or CANCEL, check if obscuring present.
+          * if From-Tag different than in CID stored, modify back CID */
+DEBUGC(DBCLASS_PROXY, "tmp+myidentlen=[%s], FromTag=[%s]",
+                         tmp+myidentlen, fromtag->gvalue);
+
+         if (strcmp(tmp+myidentlen, fromtag->gvalue) != 0) {
+            tmp[0]='\0';
+         } else { 
+            /* if From-Tag equal to in CID stored, do nothing */
+         }
+      }
+      break;
+
+   /* Outgoing Response */
+   case RESTYP_OUTGOING:
+      if (MSG_IS_RESPONSE_FOR(sipmsg,"BYE")) {
+      tmp=strstr(CID->number, myident);
+      if ((tmp==NULL)&& totag && totag->gvalue) {
+         /* no obscuring present yet, must be a new call, modify it */
+         tmp=osip_malloc(strlen(CID->number) + myidentlen + 
+                         strlen(fromtag->gvalue) + 1);
+         sprintf(tmp,"%s%s%s",CID->number,myident,totag->gvalue);
+         osip_free(CID->number);
+         CID->number=tmp;
+      } else {
+         /* Obscuring is present, do nothing. */
+      }
+}
+//      tmp=strstr(CID->number, myident);
+//      /* modify it back if existing*/
+//      if (tmp) {
+//         /* make sure to cut only the last marker - in case
+//            of multiple siproxd instances... */
+//         for (;(tmp2=strstr(tmp+1, myident)) != NULL;) {
+//            tmp=tmp2;
+//         }
+//         tmp[0]='\0';
+//      }
+      break;
+
+   }
+
+   DEBUGC(DBCLASS_PROXY, "sip_obscure_callid: new Callid#=%s",CID->number);
+
+   return STS_SUCCESS;
+}
+
 
