@@ -1,5 +1,5 @@
-/* -*- Mode: C; c-basic-offset: 3 -*-
-    Copyright (C) 2002-2005  Thomas Ries <tries@gmx.net>
+/*
+    Copyright (C) 2002-2008  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
     
@@ -25,6 +25,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <signal.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -39,7 +40,7 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: siproxd.c,v 1.67 2006/05/20 11:48:53 hb9xar Exp $";
+static char const ident[]="$Id: siproxd.c,v 1.73 2008/01/19 16:07:14 hb9xar Exp $";
 
 /* configuration storage */
 struct siproxd_config configuration;
@@ -97,6 +98,8 @@ int main (int argc, char *argv[])
    char *pidfilename=NULL;
    struct sigaction act;
 
+   log_init();
+
    log_set_stderr(1);
 
 /*
@@ -114,13 +117,15 @@ int main (int argc, char *argv[])
    if (sigaction(SIGUSR2, &act, NULL)) {
       ERROR("Failed to install SIGUSR2 handler");
    }
+   if (sigaction(SIGPIPE, &act, NULL)) {
+      ERROR("Failed to install SIGPIPE handler");
+   }
 
 
 /*
  * prepare default configuration
  */
    make_default_config();
-
    log_set_pattern(configuration.debuglevel);
 
 /*
@@ -152,7 +157,7 @@ int main (int argc, char *argv[])
       case 'c': /* load config file */
          DEBUGC(DBCLASS_CONFIG,"option: config file=%s",optarg);
          i=sizeof(configfile)-1;
-         strncpy(configfile,optarg,i-1);
+         strncpy(configfile,optarg,i);
          configfile[i]='\0';
          config_search=0;
          break; 
@@ -202,8 +207,6 @@ int main (int argc, char *argv[])
    log_set_pattern(configuration.debuglevel);
    log_set_listen_port(configuration.debugport);
 
-   /* change user and group IDs */
-   secure_enviroment();
 
    /* daemonize if requested to */
    if (configuration.daemonize) {
@@ -213,27 +216,43 @@ int main (int argc, char *argv[])
       if (fork()!=0) exit(0);
 
       log_set_stderr(0);
+
+      /* detach STDIN/OUT/ERR file */
+      i=open("/dev/null", O_RDWR);
+      close(STDIN_FILENO);
+      close(STDOUT_FILENO);
+      close(STDERR_FILENO);
+      if (dup2(i, STDIN_FILENO) <0) {
+         WARN("detaching STDIN failed: %s",strerror(errno));
+      }
+      if (dup2(i, STDOUT_FILENO) <0) {
+         WARN("detaching STDOUT failed: %s",strerror(errno));
+      }
+      if (dup2(i, STDERR_FILENO) <0) {
+         WARN("detaching STDERR failed: %s",strerror(errno));
+      }
+      close(i);
+
       INFO("daemonized, pid=%i", getpid());
    }
 
-   /* write PID file of main thread */
+   /* prepare for creating PID file */
    if (pidfilename == NULL) pidfilename = configuration.pid_file;
-   if (pidfilename) {
-      FILE *pidfile;
-      DEBUGC(DBCLASS_CONFIG,"creating PID file [%s]", pidfilename);
-      sts=unlink(configuration.pid_file);
-      if ((sts==0) ||(errno == ENOENT)) {
-         if ((pidfile=fopen(pidfilename, "w"))) {
-            fprintf(pidfile,"%i\n",(int)getpid());
-            fclose(pidfile);
-         } else {
-            WARN("couldn't create new PID file: %s", strerror(errno));
-         }
-      } else {
-         WARN("couldn't delete old PID file: %s", strerror(errno));
-      }
+
+   /* If going to dive into a chroot jail, create a PID file outside
+    * the jail, too. However, it will be owned by root and not deleted
+    * on process termination... */
+   if (configuration.chrootjail && ((getuid()==0) || (geteuid()==0))) {
+      if (pidfilename) createpidfile(pidfilename);
    }
 
+   /* change user and group IDs */
+   secure_enviroment();
+
+   /* write PID file of main thread as changed siproxd user and
+    * possibly into the chroot jail file tree  */
+   if (pidfilename) createpidfile(pidfilename);
+
    /* initialize the RTP proxy */
    sts=rtpproxy_init();
    if (sts != STS_SUCCESS) {
@@ -318,6 +337,11 @@ int main (int argc, char *argv[])
          continue; /* there are no resources to free */
       }
 
+      /*
+       * Hacks to fix-up some broken headers
+       */
+      sts=sip_fixup_asterisk(buff, &buflen);
+
       /*
        * init sip_msg
        */
@@ -369,6 +393,8 @@ int main (int argc, char *argv[])
          osip_message_get_max_forwards(ticket.sipmsg, 0, &max_forwards);
          if (max_forwards && max_forwards->hvalue) {
             forwards_count = atoi(max_forwards->hvalue);
+            if ((forwards_count<=0)||
+                (forwards_count>=LONG_MAX)) forwards_count=DEFAULT_MAXFWD;
          }
 
          DEBUGC(DBCLASS_PROXY,"checking Max-Forwards (=%i)",forwards_count);
@@ -455,6 +481,7 @@ int main (int argc, char *argv[])
 
             url = osip_message_get_uri(ticket.sipmsg);
             dest_port= (url->port)?atoi(url->port):SIP_PORT;
+            if ((dest_port <=0) || (dest_port >65535)) dest_port=SIP_PORT;
 
             if ( (get_ip_by_host(url->host, &addr1) == STS_SUCCESS) &&
                  (get_interface_ip(IF_INBOUND,&addr2) == STS_SUCCESS) &&
@@ -557,6 +584,7 @@ int main (int argc, char *argv[])
    }
 
    /* END */
+   log_end();
    return 0;
 } /* main */
 
