@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2002-2005  Thomas Ries <tries@gmx.net>
+    Copyright (C) 2002-2008  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
     
@@ -18,12 +18,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
 
-/* $Id: siproxd.h,v 1.61 2006/05/20 11:48:53 hb9xar Exp $ */
+/* $Id: siproxd.h,v 1.79 2008/01/19 16:07:14 hb9xar Exp $ */
 
 #ifdef DMALLOC
  #include <dmalloc.h>
 #endif
-
+#include <limits.h>
 
 /*
  * table to hold the client registrations
@@ -56,7 +56,7 @@ typedef struct {
  * configuration option table
  */
 struct siproxd_config {
-   int debuglevel;
+   unsigned int debuglevel;
    int debugport;
    char *inbound_if;
    char *outbound_if;
@@ -69,6 +69,8 @@ struct siproxd_config {
    int rtp_timeout;
    int rtp_dscp;
    int rtp_proxy_enable;
+   int rtp_input_dejitter;
+   int rtp_output_dejitter;
    char *user;
    char *chrootjail;
    char *hosts_allow_reg;
@@ -92,6 +94,9 @@ struct siproxd_config {
    int  pi_shortdial;
    char *pi_shortdial_akey;
    stringa_t pi_shortdial_entry;
+   char *ua_string;
+   int   use_rport;
+   int   obscure_loops;
 };
 
 /*
@@ -112,6 +117,18 @@ typedef struct {
 } sip_ticket_t;
 
 
+/*
+ * Client_ID - used to identify the two sides of a Call when one
+ * call is routed twice (in->out and back out->in) through siproxd
+ * e.g. local UA1 is calling local UA2 via an external Registrar
+ */
+#define CLIENT_ID_SIZE	128
+typedef struct {
+   char    contact[CLIENT_ID_SIZE];
+   struct  in_addr from_ip;
+   /*... maybe more to come ...*/
+} client_id_t;
+
 
 /*
  * Function prototypes
@@ -141,6 +158,7 @@ int proxy_request (sip_ticket_t *ticket);				/*X*/
 int proxy_response (sip_ticket_t *ticket);				/*X*/
 int proxy_rewrite_invitation_body(sip_ticket_t *ticket, int direction); /*X*/
 int proxy_rewrite_request_uri(osip_message_t *mymsg, int idx);		/*X*/
+int proxy_rewrite_useragent(sip_ticket_t *ticket);			/*X*/
 
 /* route_preprocessing.c */
 int route_preprocess(sip_ticket_t *ticket);				/*X*/
@@ -155,8 +173,11 @@ int  get_ip_by_host(char *hostname, struct in_addr *addr);		/*X*/
 void secure_enviroment (void);
 int  get_ip_by_ifname(char *ifname, struct in_addr *retaddr);		/*X*/
 int  get_interface_ip(int interface, struct in_addr *retaddr);		/*X*/
+int  get_interface_real_ip(int interface, struct in_addr *retaddr);	/*X*/
 char *utils_inet_ntoa(struct in_addr in);
 int  utils_inet_aton(const char *cp, struct in_addr *inp);
+int  createpidfile(char *pidfilename);					/*X*/
+int  compare_client_id(client_id_t cid1, client_id_t cid2);		/*X*/
 
 /* sip_utils.c */
 osip_message_t * msg_make_template_reply (sip_ticket_t *ticket, int code);
@@ -165,7 +186,6 @@ int  is_via_local (osip_via_t *via);					/*X*/
 int  compare_url(osip_uri_t *url1, osip_uri_t *url2);			/*X*/
 int  compare_callid(osip_call_id_t *cid1, osip_call_id_t *cid2);	/*X*/
 int  is_sipuri_local (sip_ticket_t *ticket);				/*X*/
-int  check_rewrite_rq_uri (osip_message_t *sip);			/*X*/
 int  sip_gen_response(sip_ticket_t *ticket, int code);			/*X*/
 int  sip_add_myvia (sip_ticket_t *ticket, int interface);		/*X*/
 int  sip_del_myvia (sip_ticket_t *ticket);				/*X*/
@@ -174,6 +194,8 @@ int  sip_calculate_branch_id (sip_ticket_t *ticket, char *id);		/*X*/
 int  sip_find_outbound_proxy(sip_ticket_t *ticket, struct in_addr *addr,
                              int *port);				/*X*/
 int  sip_find_direction(sip_ticket_t *ticket, int *urlidx);		/*X*/
+int  sip_fixup_asterisk(char *buff, int *buflen);			/*X*/
+int  sip_obscure_callid(sip_ticket_t *ticket);				/*X*/
 
 /* readconf.c */
 int read_config(char *name, int search);				/*X*/
@@ -181,12 +203,12 @@ int make_default_config(void);						/*X*/
 
 /* rtpproxy.c */
 int  rtpproxy_init( void );						/*X*/
-int  rtp_start_fwd (osip_call_id_t *callid, char *client_id,            /*X*/
+int  rtp_start_fwd (osip_call_id_t *callid, client_id_t client_id,	/*X*/
                     int direction, int media_stream_no,
                     struct in_addr outbound_ipaddr, int *outboundport,
-                    struct in_addr lcl_client_ipaddr, int lcl_clientport);
+                    struct in_addr lcl_client_ipaddr, int lcl_clientport,
+                    int isrtp);
 int  rtp_stop_fwd (osip_call_id_t *callid, int direction);		/*X*/
-void rtpproxy_kill( void );						/*X*/
 
 /* accessctl.c */
 int  accesslist_check(struct sockaddr_in from);
@@ -222,26 +244,36 @@ int sip_message_set_body(osip_message_t * sip, const char *buf, size_t len);
 #define DEFAULT_MAXFWD	70	/* default Max-Forward count */
 #define DEFAULT_EXPIRES	3600	/* default Expires timeout */
 
-#define URLMAP_SIZE	64	/* number of URL mapping table entries	*/
+#define URLMAP_SIZE	128	/* number of URL mapping table entries	*/
 				/* this limits the number of clients!	*/
 
-#define RTPPROXY_SIZE	128	/* number of rtp proxy entries		*/
+#define SOURCECACHE_SIZE 256	/* number of return addresses		*/
+#define DEJITTERLIMIT	1500000	/* max value for dejitter configuration */
+#define DEFAULT_DEJITTER 100000	/* default value for dejitter configuration */
+
+#define RTPPROXY_SIZE	256	/* number of rtp proxy entries		*/
 				/* this limits the number of calls!	*/
 
 #define BUFFER_SIZE	8196	/* input buffer for read from socket	*/
-#define RTP_BUFFER_SIZE	512	/* max size of an RTP frame		*/
+#define RTP_BUFFER_SIZE	1520	/* max size of an RTP frame		*/
+				/* (assume approx one Ethernet MTU)	*/
+
 #define URL_STRING_SIZE	128	/* max size of an URL/URI string	*/
-#define STATUSCODE_SIZE 5	/* size of string representation of status */
-#define DNS_CACHE_SIZE  32	/* number of entries in internal DNS cache */
-#define DNS_MAX_AGE	60	/* maximum age of an cache entry (sec)	*/
+#define STATUSCODE_SIZE	5	/* size of string representation of status */
+#define DNS_CACHE_SIZE	256	/* number of entries in internal DNS cache */
+#define DNS_ATTEMPTS	3	/* number of attempts to resolve a name
+				   before it is marked as bad */
+#define DNS_GOOD_AGE	60	/* maximum age of a good cache entry (sec) */
+#define DNS_BAD_AGE	600	/* maximum age of a bad cache entry (sec) */
 #define IFADR_CACHE_SIZE 32	/* number of entries in internal IFADR cache */
 #define IFADR_MAX_AGE	5	/* max. age of the IF address cache (sec) */
 #define IFNAME_SIZE	16	/* max string length of a interface name */
 #define HOSTNAME_SIZE	128	/* max string length of a hostname	*/
 #define USERNAME_SIZE	128	/* max string length of a username (auth) */
 #define PASSWORD_SIZE	128	/* max string length of a password (auth) */
-#define VIA_BRANCH_SIZE 128	/* max string length for via branch param */
+#define VIA_BRANCH_SIZE	128	/* max string length for via branch param */
 				/* scratch buffer for gethostbyname_r() */
+
 #if defined(PR_NETDB_BUF_SIZE)
    #define GETHOSTBYNAME_BUFLEN PR_NETDB_BUF_SIZE 
 #else
@@ -279,3 +311,26 @@ int sip_message_set_body(osip_message_t * sip, const char *buf, size_t len);
 #ifndef satoi
 #define satoi atoi  /* used in libosips MSG_TEST_CODE macro ... */
 #endif
+
+/*
+ * Macro that limits the frequency of this particular code
+ * block to no faster than every 'a' seconds. Used for logging
+ */
+#define LIMIT_LOG_RATE(a) \
+        static time_t last=0; \
+        time_t now; \
+        int dolog=0; \
+        time(&now); \
+        if ((last+(a)) <= now) {last=now; dolog=1;} \
+        if (dolog)
+
+/*
+ * if the following symbol 'GPL' is defined, building siproxd will
+ * include all features. If not defined, some features that will
+ * conflict with a non-GPL distribution license will be disabled.
+ *
+ * If you wish to distribute siproxd under another license than GPL
+ * (commercial License for example), contact the author to elaborate
+ * the details.
+ */
+#define GPL
