@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2002-2005  Thomas Ries <tries@gmx.net>
+    Copyright (C) 2002-2008  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
     
@@ -19,9 +19,11 @@
 */
 #include "config.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
+#include <limits.h>
 #include <time.h>
 #include <signal.h>
 #include <string.h>
@@ -44,7 +46,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: utils.c,v 1.43 2006/06/19 19:55:10 hb9xar Exp $";
+static char const ident[]="$Id: utils.c,v 1.53 2008/01/19 16:07:14 hb9xar Exp $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -61,8 +63,8 @@ extern int h_errno;
  *	STS_FAILURE on failure
  */
 int get_ip_by_host(char *hostname, struct in_addr *addr) {
-   int i, j;
-   time_t t;
+   int i, j, k, idx;
+   time_t t1, t2;
    struct hostent *hostentry;
 #if defined(HAVE_GETHOSTBYNAME_R)
    struct hostent result_buffer;
@@ -70,8 +72,10 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
 #endif
    int error;
    static struct {
-      time_t timestamp;
-      struct in_addr addr;
+      time_t expires_timestamp;	/* time of expiration */
+      struct in_addr addr;	/* IP address or 0.0.0.0 if a bad entry */
+      char   error_count;	/* counts failed resolution attempts */
+      char   bad_entry;		/* != 0 if resolving failed */
       char hostname[HOSTNAME_SIZE+1];
    } dns_cache[DNS_CACHE_SIZE];
    static int cache_initialized=0;
@@ -93,11 +97,11 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
       cache_initialized=1;
    }
 
-   time(&t);
+   time(&t1);
    /* clean expired entries */
    for (i=0; i<DNS_CACHE_SIZE; i++) {
       if (dns_cache[i].hostname[0]=='\0') continue;
-      if ( (dns_cache[i].timestamp+DNS_MAX_AGE) < t ) {
+      if ( (dns_cache[i].expires_timestamp) < t1 ) {
          DEBUGC(DBCLASS_DNS, "cleaning DNS cache (entry %i)", i);
          memset (&dns_cache[i], 0, sizeof(dns_cache[0]));
       }
@@ -106,17 +110,30 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
    /*
     * search requested entry in cache
     */
+   idx=0;
    for (i=0; i<DNS_CACHE_SIZE; i++) {
       if (dns_cache[i].hostname[0]=='\0') continue; /* empty */
-      if (strcmp(hostname, dns_cache[i].hostname) == 0) { /* match */
+      if (strcasecmp(hostname, dns_cache[i].hostname) == 0) { /* match */
          memcpy(addr, &dns_cache[i].addr, sizeof(struct in_addr));
+         if (dns_cache[i].bad_entry) {
+            DEBUGC(DBCLASS_DNS, "DNS lookup - blacklisted from cache: %s",
+                   hostname);
+            return STS_FAILURE;
+         }
+         if (dns_cache[i].error_count > 0) {
+            DEBUGC(DBCLASS_DNS, "DNS lookup - previous resolution failed: %s"
+                   ", attempt %i", hostname, dns_cache[i].error_count);
+            idx=i;
+            break;
+         }
          DEBUGC(DBCLASS_DNS, "DNS lookup - from cache: %s -> %s",
-	        hostname, utils_inet_ntoa(*addr));
+                hostname, utils_inet_ntoa(*addr));
          return STS_SUCCESS;
       }
    }
    
-   /* did not find it in cache, so I have to resolve it */
+   /* I did not find it in cache, so I have to resolve it */
+   error = 0;
 
    /* need to deal with reentrant versions of gethostbyname_r()
     * as we may use threads... */
@@ -155,6 +172,8 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
 #else
    #error "need gethostbyname() or gethostbyname_r()"
 #endif
+   /* Here I have 'hostentry' and 'error' */
+
 
    if (hostentry==NULL) {
       /*
@@ -162,8 +181,6 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
        * From the manpage:
        *   HOST_NOT_FOUND
        *      The specified host is unknown.
-       *   HOST_NOT_FOUND
-       *      The specified host is unknown.
        *   NO_ADDRESS or NO_DATA
        *      The requested name is valid but does not have an IP
        *      address.
@@ -186,37 +203,68 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
          ERROR("gethostbyname(%s) failed: h_errno=%i",hostname, h_errno);
 #endif
       }
-      return STS_FAILURE;
    }
 
-   memcpy(addr, hostentry->h_addr, sizeof(struct in_addr));
-   DEBUGC(DBCLASS_DNS, "DNS lookup - resolved: %s -> %s",
-          hostname, utils_inet_ntoa(*addr));
+   if (hostentry) {
+      memcpy(addr, hostentry->h_addr, sizeof(struct in_addr));
+      DEBUGC(DBCLASS_DNS, "DNS lookup - resolved: %s -> %s",
+             hostname, utils_inet_ntoa(*addr));
+   }
 
-   /*
-    * find an empty slot in the cache
-    */
-   j=0;
-   for (i=0; i<DNS_CACHE_SIZE; i++) {
-      if (dns_cache[i].hostname[0]=='\0') break;
-      if (dns_cache[i].timestamp < t) {
-         /* remember oldest entry */
-         t=dns_cache[i].timestamp;
-	 j=i;
+   /* if we already have the entry, skip finding a new empty one */
+   if (idx == 0) {
+      /*
+       * find an empty slot in the cache
+       */
+      j=0;
+      k=0;
+      t1=INT_MAX;
+      t2=INT_MAX;
+      for (i=0; i<DNS_CACHE_SIZE; i++) {
+         if (dns_cache[i].hostname[0]=='\0') break;
+         if ((dns_cache[i].expires_timestamp < t1) &&
+             (dns_cache[i].bad_entry == 0)) {
+            /* remember oldest good entry */
+            t1=dns_cache[i].expires_timestamp;
+            j=i;
+         } else 
+         if (dns_cache[i].expires_timestamp < t2) {
+            /* remember oldest bad entry */
+            t2=dns_cache[i].expires_timestamp;
+            k=i;
+         }
       }
+      /* if no empty slot found, victimize oldest one.
+       * Give preference to the oldest "bad" entry if 
+       * one exists */
+      if (i >= DNS_CACHE_SIZE) {
+         if (k > 0) i=k;
+         else       i=j;
+      }
+      idx=i;
+      memset(&dns_cache[idx], 0, sizeof(dns_cache[0]));
    }
-   /* if no empty slot found, take oldest one */
-   if (i >= DNS_CACHE_SIZE) i=j;
 
    /*
     * store the result in the cache
     */
-   DEBUGC(DBCLASS_DNS, "DNS lookup - store into cache, entry %i)", i);
-   memset(&dns_cache[i], 0, sizeof(dns_cache[0]));
-   strncpy(dns_cache[i].hostname, hostname, HOSTNAME_SIZE);
-   time(&dns_cache[i].timestamp);
-   memcpy(&dns_cache[i].addr, addr, sizeof(struct in_addr));
-
+   DEBUGC(DBCLASS_DNS, "DNS lookup - store into cache, entry %i)", idx);
+   strncpy(dns_cache[idx].hostname, hostname, HOSTNAME_SIZE);
+   dns_cache[idx].expires_timestamp = time(NULL) + DNS_GOOD_AGE;
+   if (hostentry) {
+      memcpy(&dns_cache[idx].addr, addr, sizeof(struct in_addr));
+      dns_cache[idx].error_count = 0;
+      dns_cache[idx].bad_entry = 0;
+   } else {
+      dns_cache[idx].error_count++;
+      DEBUGC(DBCLASS_DNS, "DNS lookup - errcnt=%i", dns_cache[idx].error_count);
+      if (dns_cache[idx].error_count >= DNS_ATTEMPTS) {
+         DEBUGC(DBCLASS_DNS, "DNS lookup - blacklisting entry");
+         dns_cache[idx].expires_timestamp = time(NULL) + DNS_BAD_AGE;
+         dns_cache[idx].bad_entry = 1;
+      }
+      return STS_FAILURE;
+   }
    return STS_SUCCESS;
 }
 
@@ -231,7 +279,7 @@ void secure_enviroment (void) {
    struct passwd *passwd=NULL;
 
    DEBUGC(DBCLASS_CONFIG,"running w/uid=%i, euid=%i, gid=%i, egid=%i",
-          getuid(), geteuid(), getgid(), getegid());
+          (int)getuid(), (int)geteuid(), (int)getgid(), (int)getegid());
 
    if ((getuid()==0) || (geteuid()==0)) {
       /*
@@ -270,22 +318,15 @@ void secure_enviroment (void) {
                 configuration.user);
          sts = setgid(passwd->pw_gid);
          DEBUGC(DBCLASS_CONFIG,"changed gid to %i - %s",
-	        passwd->pw_gid, (sts==0)?"Ok":"Failed");
+	        (int)passwd->pw_gid, (sts==0)?"Ok":"Failed");
 
          sts = setegid(passwd->pw_gid);
          DEBUGC(DBCLASS_CONFIG,"changed egid to %i - %s",
-	        passwd->pw_gid, (sts==0)?"Ok":"Failed");
-
-/* don't set the real user id - as we need to elevate privs
-   when setting up an RTP masquerading tunnel */
-/*&&& Actually this is no longer true (7-Jul-2004/xar) */
-//         sts = setuid(passwd->pw_uid);
-//         DEBUGC(DBCLASS_CONFIG,"changed uid to %i - %s",
-//	        passwd->pw_uid, (sts==0)?"Ok":"Failed");
+	        (int)passwd->pw_gid, (sts==0)?"Ok":"Failed");
 
          sts = seteuid(passwd->pw_uid);
          DEBUGC(DBCLASS_CONFIG,"changed euid to %i - %s",
-	        passwd->pw_uid, (sts==0)?"Ok":"Failed");
+	        (int)passwd->pw_uid, (sts==0)?"Ok":"Failed");
       }
    }
 }
@@ -294,19 +335,13 @@ void secure_enviroment (void) {
 /*
  * get_interface_ip:
  * fetches own IP address by interface INBOUND/OUTBOUND
+ * takes into account a possible outbound_host setting.
  *
  * STS_SUCCESS on returning a valid IP and interface is UP
  * STS_FAILURE if interface is DOWN or other problem
  */
 int  get_interface_ip(int interface, struct in_addr *retaddr) {
    int sts=STS_FAILURE;
-   char *tmp=NULL;
-
-      if (interface == IF_INBOUND) {
-         tmp = configuration.inbound_if;
-      } else if (interface == IF_OUTBOUND) {
-         tmp = configuration.outbound_if;
-      }
 
    if ((interface == IF_OUTBOUND) && 
               (configuration.outbound_host) &&
@@ -318,7 +353,32 @@ int  get_interface_ip(int interface, struct in_addr *retaddr) {
          sts = STS_SUCCESS;
       }
 
-   } else if (tmp && (strcmp(tmp, "")!=0)) {
+   } else  {
+      sts = get_interface_real_ip(interface, retaddr);
+   }
+
+   return sts;
+}
+
+
+/*
+ * get_interface_real_ip:
+ * fetches the real IP address of my interface INBOUND/OUTBOUND
+ *
+ * STS_SUCCESS on returning a valid IP and interface is UP
+ * STS_FAILURE if interface is DOWN or other problem
+ */
+int  get_interface_real_ip(int interface, struct in_addr *retaddr) {
+   int sts=STS_FAILURE;
+   char *tmp=NULL;
+
+      if (interface == IF_INBOUND) {
+         tmp = configuration.inbound_if;
+      } else if (interface == IF_OUTBOUND) {
+         tmp = configuration.outbound_if;
+      }
+
+   if (tmp && (strcmp(tmp, "")!=0)) {
       DEBUGC(DBCLASS_DNS, "fetching interface IP by INTERFACE [%i]", interface);
       sts = get_ip_by_ifname(tmp, retaddr);
       if (sts != STS_SUCCESS) {
@@ -508,3 +568,63 @@ int  utils_inet_aton(const char *cp, struct in_addr *inp) {
 #error "need inet_pton() or inet_aton()"
 #endif
 }
+
+/*
+ * Create the PID file
+ */
+int createpidfile(char *pidfilename) {
+   FILE *f = NULL;
+   int sts;
+   DEBUGC(DBCLASS_CONFIG,"creating PID file [%s]", pidfilename);
+   sts=unlink(pidfilename);
+   if ((sts==0) || (errno == ENOENT)) {
+      if ((f=fopen(pidfilename, "w"))) {
+         fprintf(f,"%i\n",(int)getpid());
+         fclose(f);
+      } else {
+         WARN("couldn't create new PID file: %s", strerror(errno));
+         return STS_FAILURE;
+      }
+   } else {
+      WARN("couldn't delete old PID file: %s", strerror(errno));
+      return STS_FAILURE;
+   }
+   return STS_SUCCESS;
+}
+
+
+/*
+ * compare_client_id:
+ * Compares two client_id_t structures. If both have the Contact item
+ * defined (not NULL), then compare it and return.
+ * If one (or both) do NOT have the contact item defined, then
+ * fall back on comparing the from_ip (IP address).
+ *
+ * returns:
+ * STS_SUCCESS on match
+ * STS_FAILURE on no match
+ */
+int  compare_client_id(client_id_t cid1, client_id_t cid2) {
+
+   /* Prio 1: Contact - if present in both structures */
+   if ((cid1.contact[0] != '\0') && (cid2.contact[0] != '\0')) {
+      if (strncmp(cid1.contact, cid2.contact, CLIENT_ID_SIZE) == 0) {
+         DEBUGC(DBCLASS_BABBLE, "compare_client_id: contact match [%s]",
+                cid1.contact);
+         return STS_SUCCESS;
+      }
+      DEBUGC(DBCLASS_BABBLE, "compare_client_id: contact NO match [%s<->%s]",
+             cid1.contact, cid2.contact);
+      return STS_FAILURE;
+   }
+
+   /* Prio 2: IP (always present) */
+   if (memcmp(&cid1.from_ip, &cid2.from_ip, sizeof(struct in_addr)) == 0) {
+      DEBUGC(DBCLASS_BABBLE, "compare_client_id: IP match [%s]",
+             utils_inet_ntoa(cid1.from_ip));
+      return STS_SUCCESS;
+   }
+
+   DEBUGC(DBCLASS_BABBLE, "compare_client_id: no match");
+   return STS_FAILURE;
+}
